;#########################################################################
; BIOE Module for the ATMega16 Controller
; Copyright (C) 2008 Klaus Weichinger
;
; License: CC-GNU GPL
; http://creativecommons.org/licenses/GPL/2.0/
;#########################################################################
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor,
; Boston, MA  02110-1301, USA.
;#########################################################################

;{description}
;=============================================
; DTN003 - 8bit Eingnge und 8bit Ausgnge
;=============================================
;
;  +---------------------------------+
;  |DTN 003                      AREF+-
;  |      IN          OUT        P15 +- IN 7
;  |EP2               OUT8       P14 +- IN 6
;  |EP3   IN8                    P13 +- IN 5
;  |EP4                          P12 +- IN 4
;  |EP5                          P11 +- IN 3
;  |EP6                          P10 +- IN 2
;  |EP7                          P9  +- IN 1
;  |EP8                          GND +-
;  |EP9                          P8  +- IN 0
;  |EP10                         P7  +- OUT 7
;  |EP11                         RES +-
;  |EP12                         P6  +- OUT 6
;  |EP13                         VCC +-
;  |EP14                         P5  +- OUT 5
;  |                             P4  +- OUT 4
;  |                             P3  +- OUT 3
;  |                             P2  +- OUT 2
;  |                             P1  +- OUT 1
;  |                             P0  +- OUT 0
;  +---------------------------------+
;{end}
do_DTN003:
	CPI		i,0
	BRNE	init_DTN003
	RJMP	process_DTN003
init_DTN003:
	PUSH	temp
	IN		temp,SREG
	PUSH	i

	IN		i,PORTB
	ANDI	i,~0b10100000	; Ausgnge=0 setzen
	ORI		i,0b01001101	; Eingnge PullUP's aktivieren
	OUT		PORTB,i

	IN		i,PORTD
	ANDI	i,~0b11110011	; Ausgnge=0 setzen
	ORI		i,0b00000000	; Eingnge PullUP's aktivieren
	OUT		PORTD,i

	IN		i,PORTA
	ANDI	i,~0b00000000	; Ausgnge=0 setzen
	ORI		i,0b00001111	; Eingnge PullUP's aktivieren
	OUT		PORTA,i

	IN		i,DDRB
	ORI		i,0b10100000	; Ausgnge setzen
	ANDI	i,~0b01001101	; Eingnge setzen
	OUT		DDRB,i

	IN		i,DDRD
	ORI		i,0b11110011	; Ausgnge setzen
	ANDI	i,~0b00000000	; Eingnge setzen
	OUT		DDRD,i

	IN		i,DDRA
	ORI		i,0b00000000	; Ausgnge setzen
	ANDI	i,~0b00001111	; Eingnge setzen
	OUT		DDRA,i
	
	POP		i
	OUT		SREG,temp
	POP		temp
	JMP	initDeviceType_end

process_DTN003:

	LDI		A,3
	LDI		B,0
	LDI		C,0
	SBIC 	P8_IN,P8_PIN
	ORI		C,0b00000001
	SBIC 	P9_IN,P9_PIN
	ORI		C,0b00000010
	SBIC 	P10_IN,P10_PIN
	ORI		C,0b00000100
	SBIC 	P11_IN,P11_PIN
	ORI		C,0b00001000
	SBIC 	P12_IN,P12_PIN
	ORI		C,0b00010000
	SBIC 	P13_IN,P13_PIN
	ORI		C,0b00100000
	SBIC 	P14_IN,P14_PIN
	ORI		C,0b01000000
	SBIC 	P15_IN,P15_PIN
	ORI		C,0b10000000
	RCALL	setTXEP

	LDI		A,2
	RCALL	getRXEP
	IN		i,PORTD
	ANDI	i,~0b11110011 ; Ausgnge auf 0 setzen
	SBRC	C,0
	ORI		i,0b10000000
	SBRC	C,1
	ORI		i,0b01000000
	SBRC	C,2
	ORI		i,0b00100000
	SBRC	C,3
	ORI		i,0b00010000
	SBRC	C,4
	ORI		i,0b00000010
	SBRC	C,5
	ORI		i,0b00000001
	OUT		PORTD,i

	IN		i,PORTB
	ANDI	i,~0b10100000 ; Ausgnge auf 0 setzen
	SBRC	C,6
	ORI		i,0b10000000
	SBRC	C,7
	ORI		i,0b00100000
	OUT		PORTB,i

	JMP	processDeviceType_end
