;#########################################################################
; BIOE Module for the ATMega16 Controller
; Copyright (C) 2008 Klaus Weichinger
;
; License: CC-GNU GPL
; http://creativecommons.org/licenses/GPL/2.0/
;#########################################################################
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor,
; Boston, MA  02110-1301, USA.
;#########################################################################

;{description}
;=============================================
; DTN050 - 2x10bit PWM
;          2x10bit ADC
;=============================================
;
;  +---------------------------------+
;  |DTN 050                      AREF+- 5V Out
;  |      IN          OUT        P15 +- 
;  |EP2               PWM1       P14 +-
;  |EP3               PWM2       P13 +- ADC2 In
;  |EP4               OUT        P12 +- ADC1 In
;  |EP5                          P11 +- OUT2
;  |EP6   ADC1                   P10 +- OUT1
;  |EP7   ADC2                   P9  +- OUT0
;  |EP8                          GND +-
;  |EP9                          P8  +-
;  |EP10                         P7  +-
;  |EP11                         RES +-
;  |EP12                         P6  +- Abtasttakt
;  |EP13                         VCC +-   
;  |EP14                         P5  +-
;  |                             P4  +-
;  |                             P3  +- PWM2 Out
;  |                             P2  +- PWM1 Out
;  |                             P1  +-
;  |                             P0  +-
;  +---------------------------------+
;
;  ADC1+ADC2 werden mit 15 kHz eingelesen
;  PWM-Frequenz 15.6 kHz
;  OUT=OUT0*2^0+OUT1*2^1+OUT2*2^2
;{end}
do_DTN050:
	CPI		i,0
	BRNE	init_DTN050
	RJMP	process_DTN050
init_DTN050:
	PUSH	temp
	IN		temp,SREG
	PUSH	i

	LDI		i,(1<<WGM11) | (1<<WGM10) | (1<<COM1A1) | (1<<COM1B1)
	OUT		TCCR1A,i

	LDI		i,(1<<WGM12) | (0<<CS12) | (0<<CS11) | (1<<CS10)
	OUT		TCCR1B,i


	; Ports als Ausgnge
	SBI		P2_DIR,P2_PIN
	SBI		P3_DIR,P3_PIN
	SBI		P6_DIR,P6_PIN ; Impulse Output

	SBI		P9_DIR,P9_PIN
	SBI		P10_DIR,P10_PIN
	SBI		P11_DIR,P11_PIN

	; ADC initialisieren
    LDI		i, 0 | (1<<REFS0)  ; interne Referenzspannung 5V, Kanal-0
    OUT		ADMUX, i
    LDI		i, (1<<ADEN) | (1<<ADPS2) | (0<<ADPS1) | (1<<ADPS0) ;  Division Faktor 32
    OUT		ADCSRA, i
	SBI		ADCSRA,ADSC		; ADC starten
	LDI		DEVFLAG,0

	POP		i
	OUT		SREG,temp
	POP		temp
	JMP	initDeviceType_end

process_DTN050:

	CPI		RXEPFLAG,2
	BRNE	process_DTN050_not_EP2
	LDI		RXEPFLAG,$FF	; FLAG lschen
	LDI		A,2
	RCALL	getRXEP
	OUT		OCR1AH,B
	OUT		OCR1AL,C
process_DTN050_not_EP2:

	CPI		RXEPFLAG,3
	BRNE	process_DTN050_not_EP3
	LDI		RXEPFLAG,$FF	; FLAG lschen
	LDI		A,3
	RCALL	getRXEP
	OUT		OCR1BH,B
	OUT		OCR1BL,C
process_DTN050_not_EP3:

	LDI		A,4
	RCALL	getRXEP

	IN		i,PORTB
	ANDI	i,~0b00001101 ; Ausgnge auf 0 setzen
	SBRC	C,0
	ORI		i,0b00001000
	SBRC	C,1
	ORI		i,0b00000100
	SBRC	C,2
	ORI		i,0b00000001
	OUT		PORTB,i

    SBIC	ADCSRA, ADSC
    RJMP process_DTN050_wait_adc
	; ADC ist jetzt fertig und kann verarbeitet werden

	MOV		i,DEVFLAG
 	CPI		i,0
	BRNE	process_DTN050_adc_1
process_DTN050_adc_0:
	LDI		A,6
	IN		C,ADCL
	IN		B,ADCH
	RCALL	setTXEP

    LDI		i, 1  ; Kanal-1
	MOV		DEVFLAG,i
	ORI		i,(1<<REFS0); interne Referenzspannung 5V
    OUT		ADMUX, i
	SBI		ADCSRA,ADSC		; ADC starten
	SBI		P6_OUT,P6_PIN
	RJMP	process_DTN050_adc_end

process_DTN050_adc_1:
	LDI		A,7
	IN		C,ADCL
	IN		B,ADCH
	RCALL	setTXEP

    LDI		i, 0  ; Kanal-0
	MOV		DEVFLAG,i
	ORI		i,(1<<REFS0); interne Referenzspannung 5V
    OUT		ADMUX, i
	SBI		ADCSRA,ADSC		; ADC starten
	CBI		P6_OUT,P6_PIN
	RJMP	process_DTN050_adc_end

process_DTN050_adc_end:

process_DTN050_wait_adc:

	JMP	processDeviceType_end
