;#########################################################################
; BIOE Module for the ATMega16 Controller
; Copyright (C) 2008 Klaus Weichinger
;
; License: CC-GNU GPL
; http://creativecommons.org/licenses/GPL/2.0/
;#########################################################################
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor,
; Boston, MA  02110-1301, USA.
;#########################################################################

;{description}
;=============================================
; Positionsencoder mit dem Baustein HCTL2022
;=============================================
;
;  +---------------------------------+
;  |DTN 101                      AREF+-
;  |      IN          OUT        P15 +-
;  |EP2               Position   P14 +-
;  |EP3                          P13 +-
;  |EP4                          P12 +- SEL0
;  |EP5                          P11 +- INDEX OUT  \... Zustzlicher Filter
;  |EP6                          P10 +- INDEX IN   /    fr Index (Reset)
;  |EP7                          P9  +- Kontinuierlicher CLK
;  |EP8                          GND +-
;  |EP9                          P8  +- INDEX-Led
;  |EP10                         P7  +- D7
;  |EP11                         RES +-
;  |EP12                         P6  +- D6
;  |EP13                         VCC +-
;  |EP14                         P5  +- D5
;  |                             P4  +- D4
;  |                             P3  +- D3
;  |                             P2  +- D2
;  |                             P1  +- D1
;  |                             P0  +- D0
;  +---------------------------------+
;
;{end}

do_DTN101:
	CPI		i,0
	BRNE	init_DTN101
	RJMP	process_DTN101
init_DTN101:


	PUSH	temp
	IN		temp,SREG
	PUSH	i

	; Eingnge definieren
	CBI		P0_DIR,P0_PIN
	CBI		P1_DIR,P1_PIN
	CBI		P2_DIR,P2_PIN
	CBI		P3_DIR,P3_PIN
	CBI		P4_DIR,P4_PIN
	CBI		P5_DIR,P5_PIN
	CBI		P6_DIR,P6_PIN
	CBI		P7_DIR,P7_PIN

	; PullUps aktivieren
	SBI		P0_OUT,P0_PIN
	SBI		P1_OUT,P1_PIN
	SBI		P2_OUT,P2_PIN
	SBI		P3_OUT,P3_PIN
	SBI		P4_OUT,P4_PIN
	SBI		P5_OUT,P5_PIN
	SBI		P6_OUT,P6_PIN
	SBI		P7_OUT,P7_PIN

	; INDEX IN/OUT initialisieren
	CBI		P11_OUT,P11_PIN
	SBI		P11_DIR,P11_PIN ; Ausgang

	SBI		P10_OUT,P10_PIN	; PullUP Aktivieren
	CBI		P10_DIR,P10_PIN ; Eingang

	; SEL0...SEL3 als Ausgnge setzten
	IN		i,PORTA
	ANDI	i,~0b00001111	; Ausgnge=0 setzen
	OUT		PORTA,i

	IN		i,DDRA
	ORI		i,0b00001111	; Ausgnge setzen
	OUT		DDRA,i

	LDI		i,100
	STS		S0,i

	MOV		DEVFLAG,ZERO
	MOV		DEVFLAG2,ZERO

	; INDEX-Led als Ausgang und lschen
	SBI		P8_DIR,P8_PIN
	CBI		P8_OUT,P8_PIN


	; CLK als Output setzten
	SBI		P9_DIR,P9_PIN

	; CLK erzeugen mit Timer 0
	LDI		i,(1<<WGM01)|(0<<WGM00) | (0<<COM01) | (1<<COM00) | (0<<CS02) | (0<<CS01) | (1<<CS00)
	OUT		TCCR0,i

	LDI		i,0
	OUT		OCR0,i


	SBI		P11_OUT,P11_PIN
	LDI		i,50
init_DTN101_DO_INDEX_LOOP:
	DEC		i
	CPI		i,0
	BRNE	init_DTN101_DO_INDEX_LOOP
	CBI		P11_OUT,P11_PIN

	POP		i
	OUT		SREG,temp
	POP		temp

	JMP		initDeviceType_end


process_DTN101:
	;  Positionswert lesen und auf Endpunkt ausgeben
	LDI		A,2
	CBI		P12_OUT,P12_PIN
	RCALL	DTN101_PAUSE
	MOV		B,ZERO
	SBIC 	P0_IN,P0_PIN
	ORI		B,0b00000001
	SBIC 	P1_IN,P1_PIN
	ORI		B,0b00000010
	SBIC 	P2_IN,P2_PIN
	ORI		B,0b00000100
	SBIC 	P3_IN,P3_PIN
	ORI		B,0b00001000
	SBIC 	P4_IN,P4_PIN
	ORI		B,0b00010000
	SBIC 	P5_IN,P5_PIN
	ORI		B,0b00100000
	SBIC 	P6_IN,P6_PIN
	ORI		B,0b01000000
	SBIC 	P7_IN,P7_PIN
	ORI		B,0b10000000

	SBI		P12_OUT,P12_PIN
	RCALL	DTN101_PAUSE
	MOV		C,ZERO
	SBIC 	P0_IN,P0_PIN
	ORI		C,0b00000001
	SBIC 	P1_IN,P1_PIN
	ORI		C,0b00000010
	SBIC 	P2_IN,P2_PIN
	ORI		C,0b00000100
	SBIC 	P3_IN,P3_PIN
	ORI		C,0b00001000
	SBIC 	P4_IN,P4_PIN
	ORI		C,0b00010000
	SBIC 	P5_IN,P5_PIN
	ORI		C,0b00100000
	SBIC 	P6_IN,P6_PIN
	ORI		C,0b01000000
	SBIC 	P7_IN,P7_PIN
	ORI		C,0b10000000

	CBI		P12_OUT,P12_PIN
	RCALL	DTN101_PAUSE
	MOV		i,ZERO
	SBIC 	P0_IN,P0_PIN
	ORI		i,0b00000001
	SBIC 	P1_IN,P1_PIN
	ORI		i,0b00000010
	SBIC 	P2_IN,P2_PIN
	ORI		i,0b00000100
	SBIC 	P3_IN,P3_PIN
	ORI		i,0b00001000
	SBIC 	P4_IN,P4_PIN
	ORI		i,0b00010000
	SBIC 	P5_IN,P5_PIN
	ORI		i,0b00100000
	SBIC 	P6_IN,P6_PIN
	ORI		i,0b01000000
	SBIC 	P7_IN,P7_PIN
	ORI		i,0b10000000

	CP		i,B
	BREQ	process_DTN101_continue
    
	;  Wiederholen, da in der Zwischzeit sich der Wert drastisch gendert hat
	RJMP	process_DTN101
process_DTN101_continue:	
	CALL	setTXEP


	;  INDEX (Reset) Behandlung	
	CBI		P8_OUT,P8_PIN
	LDI		i,50
process_DTN101_INDEX_LOOP_ON:
	SBIS	P10_IN,P10_PIN
	JMP		processDeviceType_end
	NOP
	NOP
	DEC		i
	CPI		i,0
	BRNE	process_DTN101_INDEX_LOOP_ON
	;  INDEX liegt bereits eine weile an (ohne Aussetzer)
	
	CPI		DEVFLAG2,0
	BREQ	process_DTN101_DO_INDEX  ;  Wenn 1. INDEX, dann sofort erlauben
	LDS		i,S0
	MOV		DEVDATH,B
	MOV		DEVDATL,C
	CP		C,i
	CPC		B,ZERO
	BRLO	process_DTN101_DO_INDEX

	COM		C
	COM		B
	ADD		C,ONE
	ADC		B,ZERO

	CP		C,i
	CPC		B,ZERO
	BRLO	process_DTN101_DO_INDEX

	JMP		processDeviceType_end
		
process_DTN101_DO_INDEX:
	SBI		P8_OUT,P8_PIN

	LDI		DEVFLAG,1		;  Flag fr 1. INDEX(Reset) setzen
	SBI		P11_OUT,P11_PIN
	LDI		i,50
process_DTN101_DO_INDEX_LOOP:
	DEC		i
	CPI		i,0
	BRNE	process_DTN101_DO_INDEX_LOOP
	CBI		P11_OUT,P11_PIN	
	JMP		processDeviceType_end

DTN101_PAUSE:
	PUSH	i
	LDI		i,20
DTN101_sendPAUSE_wait:
	NOP
	DEC		i
	CPI		i,0
	BRNE	DTN101_sendPAUSE_wait
	POP		i
	RET

	


