;#########################################################################
; BIOE Module for the ATMega16 Controller
; Copyright (C) 2008 Klaus Weichinger
;
; License: CC-GNU GPL
; http://creativecommons.org/licenses/GPL/2.0/
;#########################################################################
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor,
; Boston, MA  02110-1301, USA.
;#########################################################################

;{description}
;=============================================
; Infrarot RC5 Empfnger
;=============================================
;
;  +---------------------------------+
;  |DTN 120                      AREF+-
;  |      IN          OUT        P15 +-
;  |EP2               Data       P14 +-
;  |EP3                          P13 +-
;  |EP4                          P12 +-
;  |EP5                          P11 +- 
;  |EP6                          P10 +- IR-Receiver
;  |EP7                          P9  +- LED
;  |EP8                          GND +-
;  |EP9                          P8  +- 
;  |EP10                         P7  +-
;  |EP11                         RES +-
;  |EP12                         P6  +-
;  |EP13                         VCC +-
;  |EP14                         P5  +-
;  |                             P4  +-
;  |                             P3  +-
;  |                             P2  +-
;  |                             P1  +-
;  |                             P0  +-
;  +---------------------------------+
;{end}

do_DTN120:
	CPI		i,0
	BRNE	init_DTN120
	RJMP	process_DTN120
init_DTN120:


	PUSH	temp
	IN		temp,SREG
	PUSH	i
	CBI		P10_DIR,P10_PIN ; P10 als Eingang (INT2, flanken aktiv) 
	SBI		P10_OUT,P10_PIN ; PullUP aktivieren

	SBI		P9_DIR,P9_PIN
	CBI		P9_OUT,P9_PIN

	MOV		DEVDATL,ZERO	; RC5Command
	MOV		DEVDATH,ZERO	; RC5Device
	MOV		DEVDATX,ZERO	; RC5Count
	RCALL	DTN120_Reset
	POP		i
	OUT		SREG,temp
	POP		temp

	JMP		initDeviceType_end

process_DTN120:
	JMP	processDeviceType_end

DTN120_Reset:
	IN		i,TIMSK		; Timer Interrupt Disable (Overflow)
	ANDI	i,~(1<<TOIE2)
	OUT		TIMSK,i


	MOV		DEVDATL,ZERO
	MOV		DEVDATH,ZERO
	LDI		DEVFLAG,13 ; RC5PosCount
	MOV		DEVFLAG2,ONE ; RC5TimerCount=1
	MOV		DEVDATY,ZERO  ; RC5Temp


	IN		i,MCUCSR		; INT2 aktiv auf fallende Flanke
	ANDI	i,~(1<<ISC2)
	OUT		MCUCSR,i

	IN		i,GIFR
	ORI		i,(1<<INTF2)	; INT2 Flag lschen
	OUT		GIFR,i

	IN		i,GICR
	ORI		i,(1<<INTF2)	; INT2 aktivieren
	OUT		GICR,i

	RET
