;#########################################################################
; BIOE Module for the ATMega16 Controller
; Copyright (C) 2008 Klaus Weichinger
;
; License: CC-GNU GPL
; http://creativecommons.org/licenses/GPL/2.0/
;#########################################################################
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor,
; Boston, MA  02110-1301, USA.
;#########################################################################

;{description}
;=============================================
; WII-Nunchuk Anbindung
;=============================================
;
;  +---------------------------------+
;  |DTN 128                      AREF+-
;  |      IN          OUT        P15 +-
;  |EP2               Distance   P14 +-
;  |EP3                          P13 +-
;  |EP4                          P12 +-
;  |EP5                          P11 +- SDA ----+-------
;  |EP6                          P10 +- SCL ----|---+---
;  |EP7                          P9  +- LED    +++ +++
;  |EP8                          GND +-        | | | |
;  |EP9                          P8  +-    4k7 | | | | 4k7
;  |EP10                         P7  +-        | | | |
;  |EP11                         RES +-        +++ +++
;  |EP12                         P6  +-         |   |
;  |EP13                         VCC +----------+---+
;  |EP14                         P5  +-
;  |                             P4  +-
;  |                             P3  +-
;  |                             P2  +-
;  |                             P1  +-
;  |                             P0  +-
;  +---------------------------------+
;{end}

do_DTN128:
	CPI		i,0
	BRNE	init_DTN128
	RJMP	process_DTN128
init_DTN128:


	PUSH	temp
	IN		temp,SREG
	PUSH	i

	; LED initialisieren
	SBI		P9_DIR,P9_PIN
	CBI		P9_OUT,P9_PIN

	MOV		DEVDATL,ZERO ; NUNCHUK present if !=0
	LDI		A,10
	RCALL	i2c_init

	POP		i
	OUT		SREG,temp
	POP		temp
	JMP		initDeviceType_end

process_DTN128:
	DEC		DEVDATH
	MOV		i,DEVDATH
	CPI		i,0
	BREQ	process_DTN128_continue
	RCALL	nunchuck_pause
	JMP	processDeviceType_end
process_DTN128_continue:
	MOV		i,DEVDATL
	CPI		i,0
	BRNE	process_DTN128_NUNCHEK_read
	CBI		P9_OUT,P9_PIN
	; INIT NUNCHUK
	LDI		A,10
	RCALL	i2c_start
	LDI		B,0xA4
	RCALL	i2c_write
	MOV		DEVDATL,B
	LDI		B,0x40
	RCALL	i2c_write
	LDI		B,0x00
	RCALL	i2c_write
	RCALL	i2c_stop
	JMP	processDeviceType_end




process_DTN128_NUNCHEK_read:
	SBI		P9_OUT,P9_PIN
	LDI		A,10

	RCALL	i2c_start
	LDI		B,0xA5
	RCALL	i2c_write
	MOV		DEVDATL,B
	RCALL	i2c_read_ACK	; BYTE0
	RCALL	decode_nunchuk
	STS		B0,B
	RCALL	i2c_read_ACK	; BYTE1
	RCALL	decode_nunchuk
	STS		B1,B
	RCALL	i2c_read_ACK	; BYTE2
	RCALL	decode_nunchuk
	STS		B2,B
	RCALL	i2c_read_ACK	; BYTE3
	RCALL	decode_nunchuk
	STS		B3,B
	RCALL	i2c_read_ACK	; BYTE4
	RCALL	decode_nunchuk
	STS		B4,B
	RCALL	i2c_read_NACK	; BYTE5
	RCALL	decode_nunchuk
	STS		B5,B
	RCALL	i2c_stop

	LDI		A,2
	LDI		B,0
	LDS		C,B0
	CALL	setTXEP

	LDI		A,3
	LDI		B,0
	LDS		C,B1
	CALL	setTXEP
	
	LDS		i,B5

	LDI		A,6 // Z-Acceleration
	LDI		B,0
	LDS		C,B4
	ROL		i
	ROL		C
	ROL		B
	ROL		i
	ROL		C
	ROL		B
	CALL	setTXEP

	LDI		A,5 // Y-Acceleration
	LDI		B,0
	LDS		C,B3	
	ROL		i
	ROL		C
	ROL		B
	ROL		i
	ROL		C
	ROL		B
	CALL	setTXEP

	LDI		A,4 // X-Acceleration
	LDI		B,0
	LDS		C,B2
	ROL		i
	ROL		C
	ROL		B
	ROL		i
	ROL		C
	ROL		B
	CALL	setTXEP

	LDI		A,7
	LDI		B,0
	LDS		C,B5
	ANDI	C,0x03
	CALL	setTXEP

	RCALL	nunchuck_pause

	RCALL	i2c_start
	LDI		B,0xA4
	RCALL	i2c_write
	LDI		B,0
	RCALL	i2c_write
	RCALL	i2c_stop

	JMP	processDeviceType_end


nunchuck_pause:
	PUSH	temp
	IN		temp,SREG
	PUSH	i
	LDI		i,100
nunchuck_pause_loop:
	NOP
	NOP
	NOP
	DEC		i
	CPI		i,0
	BRNE	nunchuck_pause_loop
	POP		i
	OUT		SREG,temp
	POP		temp
	RET


decode_nunchuk:
	PUSH	temp
	IN		temp,SREG
	PUSH	i
	LDI		i,0x17
	EOR		B,i
	ADD		B,i
	POP		i
	OUT		SREG,temp
	POP		temp
	RET

; A ... I2C-Speedfaktor
; B ... Value
; C ... ACK
i2c_pause:
	PUSH	temp
	IN		temp,SREG
	PUSH	i
	LDI		i,30
i2c_pause_loop:
	NOP
	NOP
	NOP
	DEC		i
	CPI		i,0
	BRNE	i2c_pause_loop
	POP		i
	OUT		SREG,temp
	POP		temp
	RET

i2c_init:
    ; SCL - Eingang -> mit PullUP=High
	CBI		P10_DIR,P10_PIN
	CBI		P10_OUT,P10_PIN    
    ; SDA - Eingang -> mit PullUP=High
	CBI		P11_DIR,P11_PIN
	CBI		P11_OUT,P11_PIN    
	RET

i2c_start:
	CBI		P11_DIR,P11_PIN ; SDA=1
	CBI		P10_DIR,P10_PIN	; SCL=1
	RCALL	i2c_pause
	SBI		P11_DIR,P11_PIN ; SDA=0
	RCALL	i2c_pause
	SBI		P10_DIR,P10_PIN ; SCL=0
	RCALL	i2c_pause
	RET

i2c_stop:
	SBI		P11_DIR,P11_PIN ; SDA=0
	RCALL	i2c_pause
	CBI		P10_DIR,P10_PIN	; SCL=1
	RCALL	i2c_pause
	CBI		P11_DIR,P11_PIN ; SDA=1
	RCALL	i2c_pause
	RET

i2c_write:
	PUSH	temp
	IN		temp,SREG
	PUSH	i
	LDI		i,8
i2c_write_loop:	
	LSL		B
	BRCC	i2c_write_0
	CBI		P11_DIR,P11_PIN ; SDA=1
	JMP		i2c_write_continue
i2c_write_0:
	SBI		P11_DIR,P11_PIN ; SDA=0
i2c_write_continue:	
	RCALL	i2c_pause
	CBI		P10_DIR,P10_PIN ; SCL=1
	RCALL	i2c_pause
	SBI		P10_DIR,P10_PIN ; SCL=0
	RCALL	i2c_pause
	DEC		i
	CPI		i,0
	BRNE	i2c_write_loop
	LDI		B,0
; ACK lesen
	CBI		P11_DIR,P11_PIN
	RCALL	i2c_pause
	CBI		P10_DIR,P10_PIN ; SCL=1
	RCALL	i2c_pause
	SBIS	P11_IN,P11_PIN
	LDI		B,1
	SBI		P10_DIR,P10_PIN ; SCL=0
	RCALL	i2c_pause
	SBI		P11_DIR,P11_PIN ; SDA=0
	RCALL	i2c_pause
	POP		i
	OUT		SREG,temp
	POP		temp
	RET

i2c_read_ACK:
	PUSH	temp
	IN		temp,SREG
	PUSH	i
	CBI		P11_DIR,P11_PIN ; SDA=1
	LDI		i,8
i2c_read_ACK_loop:	
	RCALL	i2c_pause
	CBI		P10_DIR,P10_PIN ; SCL=1
	RCALL	i2c_pause
	CLC
	SBIC	P11_IN,P11_PIN
	SEC
	ROL		B
	SBI		P10_DIR,P10_PIN ; SCL=0
	RCALL	i2c_pause
	DEC		i
	CPI		i,0
	BRNE	i2c_read_ACK_loop
; ACK schreiben
	SBI		P11_DIR,P11_PIN ; SDA=0
	RCALL	i2c_pause
	CBI		P10_DIR,P10_PIN ; SCL=1
	RCALL	i2c_pause
	SBI		P10_DIR,P10_PIN ; SCL=0
	RCALL	i2c_pause
	SBI		P11_DIR,P11_PIN ; SDA=0
	POP		i
	OUT		SREG,temp
	POP		temp
	RET

i2c_read_NACK:
	PUSH	temp
	IN		temp,SREG
	PUSH	i
	CBI		P11_DIR,P11_PIN ; SDA=1
	LDI		i,8
i2c_read_NACK_loop:	
	RCALL	i2c_pause
	CBI		P10_DIR,P10_PIN ; SCL=1
	RCALL	i2c_pause
	CLC
	SBIC	P11_IN,P11_PIN
	SEC
	ROL		B
	SBI		P10_DIR,P10_PIN ; SCL=0
	RCALL	i2c_pause
	DEC		i
	CPI		i,0
	BRNE	i2c_read_NACK_loop
; NACK schreiben
	CBI		P11_DIR,P11_PIN
	RCALL	i2c_pause
	CBI		P10_DIR,P10_PIN ; SCL=1
	RCALL	i2c_pause
	SBI		P10_DIR,P10_PIN ; SCL=0
	RCALL	i2c_pause
	SBI		P11_DIR,P11_PIN ; SDA=0
	RCALL	i2c_pause
	POP		i
	OUT		SREG,temp
	POP		temp
	RET
