;#########################################################################
; BIOE Module for the ATMega16 Controller
; Copyright (C) 2008 Klaus Weichinger
;
; License: CC-GNU GPL
; http://creativecommons.org/licenses/GPL/2.0/
;#########################################################################
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor,
; Boston, MA  02110-1301, USA.
;#########################################################################

;{description}
;=============================================
; Streckensimulator SLAVE
; mit synchroner serieller Schnittstelle
;=============================================
;
;  +---------------------------------+
;  |DTN 201                      AREF+-
;  |      IN          OUT        P15 +- 
;  |EP2                          P14 +- 
;  |EP3                          P13 +- 
;  |EP4                          P12 +- 
;  |EP5                          P11 +- CLK (Input)
;  |EP6                          P10 +- CA (Communication active=HIGH)
;  |EP7                          P9  +- 
;  |EP8                          GND +-
;  |EP9                          P8  +- 
;  |EP10                         P7  +- 
;  |EP11                         RES +-
;  |EP12                         P6  +- 
;  |EP13                         VCC +-
;  |EP14                         P5  +- RXD
;  |                             P4  +- TXD
;  |                             P3  +- 
;  |                             P2  +- 
;  |                             P1  +- 
;  |                             P0  +- 
;  +---------------------------------+
;
;  S0 ... BIOE-Bus Communication active Watchdog Time (in ms) (default 10ms)
;
;{end}

do_DTN201:
	CPI		i,0
	BRNE	init_DTN201
	RJMP	process_DTN201
init_DTN201:

	PUSH	temp
	IN		temp,SREG
	PUSH	i

    SBI		P10_DIR,P10_PIN
    CBI		P10_OUT,P10_PIN


    ; Frame-Format: 8 Bit
    CBI		P11_DIR, P11_PIN		; Slave mode
	LDI 	i, (1<<UMSEL)|(0<<UCPOL)|(3<<UCSZ0)|(1<<URSEL)
    OUT 	UCSRC, i

    SBI 	UCSRB, TXEN 			; USRT transmit enable
	SBI 	UCSRB, RXEN             ; USRT receive enable


	LDI		i,0x4E		;  Timer erzeugt alle 1ms einen Overflow
	OUT		ICR1H,i
	LDI		i,0x00
	OUT		ICR1L,i
	
	LDI		i,0
	MOV		DEVDATX,i	;  Register fr den Software-WatchDog

	LDI		i,(1<<WGM11) | (0<<WGM10) | (0<<COM1A1) | (0<<COM1A0) | (0<<COM1B1) | (0<<COM1B0)
	OUT		TCCR1A,i

	LDI		i,(1<<WGM13)|(1<<WGM12) | (0<<CS12) | (0<<CS11) | (1<<CS10)
	OUT		TCCR1B,i

	IN		i,TIMSK
	ORI		i,(1<<TOIE1)
	OUT		TIMSK,i

	LDI		i,10
	STS		S0,i

	POP		i
	OUT		SREG,temp
	POP		temp

	JMP	initDeviceType_end


process_DTN201:

	MOV		i,DEVDATX
	CPI		i,0
	BRNE	process_DTN201_NOT_WATCHDOG
    CBI		P10_OUT,P10_PIN
	RJMP	process_DTN201_END_WATCHDOG
process_DTN201_NOT_WATCHDOG:
    SBI		P10_OUT,P10_PIN
process_DTN201_END_WATCHDOG:

; berprfung, ob am EP aktivitten seitens BIOE-Interface waren

	CPI		RXEPFLAG,2
	BRLO	process_DTN201_NOEPAKTION
	CPI		RXEPFLAG,15
	BRSH	process_DTN201_NOEPAKTION

	LDI		RXEPFLAG,$FF	; FLAG lschen
	LDS		DEVDATX,S0		; Watchdog fr CA-Pin Reseten
process_DTN201_NOEPAKTION:

	SBIS 	UCSRA, RXC              ; return if nothing was received
	JMP	processDeviceType_end

    
	IN 		i, UDR 					; read received Byte from UART Data register              

 	CPI 	i, 82					; R received
	brne	process_DTN201_not_R
	LDI		DEVFLAG, 1
	JMP	processDeviceType_end
process_DTN201_not_R:								

	CPI 	i, 87					; W received
	BRNE	process_DTN201_not_W

	LDI		DEVFLAG,11
	JMP	processDeviceType_end
process_DTN201_not_W:


	
	CPI		DEVFLAG, 1
	BRNE	process_DTN201_not_read_EP

	RCALL	DTN201_convert_to_Num
	MOV		A,i
	CALL	getRXEP


	; send four high bits of B
	MOV		i,B
	SWAP	i
	ANDI	i, 0b00001111
	RCALL	DTN201_sendPAUSE
	RCALL	DTN201_sendASCII

	; send four low bits of B
	MOV		i,B
	ANDI	i, 0b00001111
	RCALL	DTN201_sendPAUSE
	RCALL	DTN201_sendASCII
	
	; send four high bits of C
	MOV		i,C
	SWAP	i
	ANDI	i, 0b00001111
	RCALL	DTN201_sendPAUSE
	RCALL	DTN201_sendASCII

	; send four low bits of C
	MOV		i,C
	ANDI	i, 0b00001111
	RCALL	DTN201_sendPAUSE
	RCALL	DTN201_sendASCII

	MOV		DEVFLAG, ZERO		; work done
	JMP	processDeviceType_end

process_DTN201_not_read_EP:

	CPI		DEVFLAG, 11				; EP Number was send
	BRNE	process_DTN201_not_write_EP
	RCALL	DTN201_convert_to_Num
	MOV		DEVFLAG2,i				; copy EP number to DEVFLAG2 register
	LDI		DEVFLAG, 12
	JMP	processDeviceType_end
process_DTN201_not_write_EP:

	CPI		DEVFLAG, 12				; first of four Data bytes was send
	BRNE	process_DTN201_not_ASCII_1
	RCALL	DTN201_convert_to_Num
	SWAP	i
	ANDI	i, 0b11110000
	MOV		DEVDATH, i
	LDI		DEVFLAG, 13
	JMP	processDeviceType_end
process_DTN201_not_ASCII_1:

	CPI		DEVFLAG, 13				; second of four Data bytes was send
	BRNE	process_DTN201_not_ASCII_2
	RCALL	DTN201_convert_to_Num
	ANDI	i, 0b00001111
	OR		DEVDATH, i
	LDI		DEVFLAG, 14
	JMP	processDeviceType_end
process_DTN201_not_ASCII_2:

	CPI		DEVFLAG, 14				; third of four Data bytes was send
	BRNE	process_DTN201_not_ASCII_3
	RCALL	DTN201_convert_to_Num
	SWAP	i
	ANDI	i, 0b11110000
	MOV		DEVDATL, i
	LDI		DEVFLAG, 15
	JMP	processDeviceType_end
process_DTN201_not_ASCII_3:

	CPI		DEVFLAG, 15				; fourth of four Data bytes was send
	BRNE	process_DTN201_not_ASCII_4
	RCALL	DTN201_convert_to_Num
	ANDI	i, 0b00001111
	OR		DEVDATL, i
	MOV 	A, DEVFLAG2
	MOV		B, DEVDATH
	MOV		C, DEVDATL
	CALL	setTXEP					; write Data to EP
	MOV		DEVFLAG, ZERO

	JMP	processDeviceType_end
process_DTN201_not_ASCII_4:

	JMP	processDeviceType_end

DTN201_sendASCII:
	CPI		i,10
	BRSH	DTN201_ABCDEF
	SUBI	i,-48
	RJMP	DTN201_weiter_r
DTN201_ABCDEF:
	SUBI	i,-55
DTN201_weiter_r:
DTN201_UARTsend:
    SBIS UCSRA,UDRE                  ; wait until UDR is ready for next byte
    RJMP DTN201_UARTsend
    OUT UDR, i
    RET

DTN201_sendPAUSE:
	PUSH	i
	LDI		i,20
DTN201_sendPAUSE_wait:
	DEC		i
	CPI		i,0
	BRNE	DTN201_sendPAUSE_wait
	POP		i
	RET

DTN201_convert_to_Num:
	CPI		i, 65
	BRLO	DTN201_NUMBER
	SUBI	i,55
	RJMP 	DTN201_weiter_w
DTN201_NUMBER:
	SUBI	i, 48
DTN201_weiter_w:
	RET
	


