;#########################################################################
; BIOE Module for the ATMega16 Controller
; Copyright (C) 2008 Klaus Weichinger
;
; License: CC-GNU GPL
; http://creativecommons.org/licenses/GPL/2.0/
;#########################################################################
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor,
; Boston, MA  02110-1301, USA.
;#########################################################################

;   #################################
;   INT2 Handler
;   #################################

int2_handler:
	SEI
	PUSH	temp
	IN		temp,SREG
	PUSH	i

	LDS		i,DTN
	CPI		i,100
	BRNE	int2_handler_not_DTN100
	RJMP	int2_handler_DTN100
int2_handler_not_DTN100:

	CPI		i,106
	BRNE	int2_handler_not_DTN106
	RJMP	int2_handler_DTN100
int2_handler_not_DTN106:

	LDS		i,DTN
	CPI		i,91
	BRNE	int2_handler_not_DTN091
	RJMP	int2_handler_DTN091
int2_handler_not_DTN091:

	LDS		i,DTN
	CPI		i,120
	BRNE	int2_handler_not_DTN120
	RJMP	int2_handler_DTN120
int2_handler_not_DTN120:

	LDS		i,DTN
	CPI		i,122
	BRNE	int2_handler_not_DTN122
	RJMP	int2_handler_DTN122
int2_handler_not_DTN122:

	POP		i
	OUT		SREG,temp
	POP		temp
	RETI


int2_handler_DTN091:
	CPI		DEVFLAG,255
	BREQ	int2_handler_DTN091_WAS_0Hz
	INC		DEVFLAG
	CPI		DEVFLAG,4
	BRNE	int2_handler_DTN091_NOT_4_Periodes
	IN		DEVDATL,TCNT1L
	IN		DEVDATH,TCNT1H

int2_handler_DTN091_WAS_0Hz:
	OUT		TCNT1H,ZERO
	OUT		TCNT1L,ZERO
	MOV		DEVFLAG,ZERO
int2_handler_DTN091_NOT_4_Periodes:

	POP		i
	OUT		SREG,temp
	POP		temp
	RETI


int2_handler_DTN100:

	PUSH	j
	;  Aktive Flanke fr INT2 togglen
	IN		i,MCUCSR
	LDI		j,(1<<ISC2)
	EOR		i,j
	OUT		MCUCSR,i
	POP		j

	IN		i,PINB	;  Encoder-Leitungen lesen
	ANDI	i,0b00001101

	PUSH	i
	ANDI	i,0b00000001
	CPI		i,0b00000001
	BRNE	int2_handler_DTN100_not_RESET
	LDI		i,0x80
;	CP		DEVFLAG2,ZERO
;	BREQ	int2_handler_DTN100_doRESET
;	CP		DEVDATH,i
;	BRNE	int2_handler_DTN100_not_RESET
; int2_handler_DTN100_doRESET:
;	MOV		DEVFLAG2,ZERO
	MOV		DEVDATH,i
	MOV		DEVDATL,zero
	POP		i
	RJMP	int2_handler_DTN100_end
int2_handler_DTN100_not_RESET:
	POP		i

	ANDI	i,0b00001100
	CPI		i,0b00001100
	BRNE	int2_handler_DTN100_not_11
	SUB		DEVDATL,ONE
	SBC		DEVDATH,ZERO
	RJMP	int2_handler_DTN100_end
int2_handler_DTN100_not_11:

	CPI		i,0b00000000
	BRNE	int2_handler_DTN100_not_00
	SUB		DEVDATL,ONE
	SBC		DEVDATH,ZERO
	RJMP	int2_handler_DTN100_end
int2_handler_DTN100_not_00:

	ADD		DEVDATL,one
	ADC		DEVDATH,zero

int2_handler_DTN100_end:
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI


int2_handler_DTN120:
	IN		i,GICR
    ANDI	i,~(1<<INT2)	; INT2 Deaktivieren
	OUT		GICR, i	

	SBI		P9_OUT,P9_PIN

	MOV		DEVFLAG2,ONE ; RC5TimerCount=1
	DEC		DEVFLAG
	LDI		i,(1<<CS22)|(1<<CS21)|(1<<CS20); ;  1024 Prescaler
	OUT		TCCR2,i

	LDI		i,240
	OUT		TCNT2,i

	LDI		i,(1<<TOV2) ; Lschen des Timer2 Overflow Flags
	OUT		TIFR,i

	IN		i,TIMSK		; Timer Interrupt Enable (Overflow)
	ORI		i,(1<<TOIE2)
	OUT		TIMSK,i

	POP		i
	OUT		SREG,temp
	POP		temp
	RETI

int2_handler_DTN122:
; Interrupt fr die PS2-Kommunikation

	PUSH	j
	MOV		i,DEVDATY
	ANDI	i,0b01000000
	CPI		i,0
	BRNE	int2_handler_DTN122_senddata
	; Daten werden empfangen
	MOV		j,ZERO
	SBIC	P4_IN,P4_PIN
	MOV		j,ONE
	MOV		i,DEVDATY
	ANDI	i,0x0f
	CPI		i,1
	BRNE	int2_handler_DTN122_getdata_not_STOP
	CPI		j,1
	BRNE	int2_handler_DTN122_getdata_error
	MOV		DEVDATL,DEVDATX
	LDI		i,0x0b
	MOV		DEVDATY,i
	CBI		P9_OUT,P9_PIN
	RJMP	int2_handler_DTN122_end
int2_handler_DTN122_getdata_not_STOP:

	CPI		i,2
	BRNE	int2_handler_DTN122_getdata_not_PARITY
	DEC		DEVDATY
	MOV		i,DEVDATY
	ANDI	i,0b10000000
	ROR		j
	ROR		j
	ANDI	j,0b10000000
	CP		i,j
	BREQ	int2_handler_DTN122_getdata_error
	RJMP	int2_handler_DTN122_end
int2_handler_DTN122_getdata_not_PARITY:

	CPI		i,0x0b
	BRNE	int2_handler_DTN122_getdata_not_START
	CPI		j,0
	BRNE	int2_handler_DTN122_getdata_error
	MOV		DEVDATX,ZERO
	DEC		DEVDATY
;	SBI		P9_OUT,P9_PIN
	RJMP	int2_handler_DTN122_end
int2_handler_DTN122_getdata_not_START:

	CPI		j,0
	BREQ	int2_handler_DTN122_getdata_0
	MOV		i,DEVDATY
	SUBI	i,-128
	MOV		DEVDATY,i
int2_handler_DTN122_getdata_0:

	ROR		j
	ROR		DEVDATX
	DEC		DEVDATY
	RJMP	int2_handler_DTN122_end

int2_handler_DTN122_getdata_error:
	LDI		i,0x0b
	MOV		DEVDATY,i
	CBI		P5_DIR,P5_PIN		; CLK=1
	CBI		P4_DIR,P4_PIN		; DATA=1

	RJMP	int2_handler_DTN122_end

;==============================================================
int2_handler_DTN122_senddata:
	; Daten werden gesendet

	MOV		i,DEVDATY
	ANDI	i,0b00001111

	CPI		i,1
	BRNE	int2_handler_DTN122_senddata_not_ACK
	MOV		j,ZERO
	SBIC	P4_IN,P4_PIN
	MOV		j,ONE
	CP		j,ZERO
	BRNE	int2_handler_DTN122_senddata_error
	LDI		i,0x0b
	MOV		DEVDATY,i
	CBI		P9_OUT,P9_PIN
	RJMP	int2_handler_DTN122_end
int2_handler_DTN122_senddata_not_ACK:

	CPI		i,2
	BRNE	int2_handler_DTN122_senddata_not_STOP
	DEC		DEVDATY
	CBI		P4_DIR,P4_PIN		; DATA=1
	RJMP	int2_handler_DTN122_end
int2_handler_DTN122_senddata_not_STOP:

	CPI		i,3
	BRNE	int2_handler_DTN122_senddata_not_PARITY
	DEC		DEVDATY
	MOV		i,DEVDATX
	ANDI	i,0b10000000
	CPI		i,0
	BRNE	int2_handler_DTN122_senddata_PARITY1
	SBI		P4_DIR,P4_PIN		; DATA=1
	RJMP	int2_handler_DTN122_end
int2_handler_DTN122_senddata_PARITY1:
	CBI		P4_DIR,P4_PIN		; DATA=0
	RJMP	int2_handler_DTN122_end
int2_handler_DTN122_senddata_not_PARITY:

	CPI		i,12
	BRNE	int2_handler_DTN122_senddata_not_START
	DEC		DEVDATY
;	SBI		P9_OUT,P9_PIN
	RJMP	int2_handler_DTN122_end
int2_handler_DTN122_senddata_not_START:

	; Datenbits werden gesendet
	ROR		DEVDATX
	BRCS	int2_handler_DTN122_senddata_CSET
	SBI		P4_DIR,P4_PIN		; DATA=0
	JMP		int2_handler_DTN122_senddata_continue
int2_handler_DTN122_senddata_CSET:
	CBI		P4_DIR,P4_PIN		; DATA=1
	MOV		i,DEVDATY
	SUBI	i,-128
	MOV		DEVDATY,i
int2_handler_DTN122_senddata_continue:
	DEC		DEVDATY

	RJMP	int2_handler_DTN122_end

int2_handler_DTN122_senddata_error:
	LDI		i,0x0b
	MOV		DEVDATY,i
	CBI		P5_DIR,P5_PIN		; CLK=1
	CBI		P4_DIR,P4_PIN		; DATA=1

int2_handler_DTN122_end:
	POP		j
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI

;   #################################
;   TIMER1 Overflow Handler
;   #################################

timer1_overflow:
	SEI
	PUSH	temp
	IN		temp,SREG
	PUSH	i

	LDS		i,DTN
	CPI		i,91
	BRNE	timer1_overflow_not_DTN091
	RJMP	timer1_overflow_DTN091
timer1_overflow_not_DTN091:

	LDS		i,DTN
	CPI		i,103
	BRNE	timer1_overflow_not_DTN103
	RJMP	timer1_overflow_DTN103
timer1_overflow_not_DTN103:

	LDS		i,DTN
	CPI		i,104
	BRNE	timer1_overflow_not_DTN104
	RJMP	timer1_overflow_DTN104
timer1_overflow_not_DTN104:

	LDS		i,DTN
	CPI		i,106
	BRNE	timer1_overflow_not_DTN106
	RJMP	timer1_overflow_DTN106
timer1_overflow_not_DTN106:

	LDS		i,DTN
	CPI		i,127
	BRNE	timer1_overflow_not_DTN127
	RJMP	timer1_overflow_DTN106
timer1_overflow_not_DTN127:

	LDS		i,DTN
	CPI		i,201
	BRNE	timer1_overflow_not_DTN201
	RJMP	timer1_overflow_DTN201
timer1_overflow_not_DTN201:

	POP		i
	OUT		SREG,temp
	POP		temp
	RETI

timer1_overflow_DTN091:
	LDI		DEVFLAG,255
	LDI		i,0xFF
	MOV		DEVDATL,i
	MOV		DEVDATH,i
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI

timer1_overflow_DTN103:
	MOV		i,DEVDATY
	ANDI	i,0b01111111
	CP		i,ZERO
	BREQ	timer1_overflow_DTN103_zero
	DEC		DEVDATY
timer1_overflow_DTN103_zero:
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI

timer1_overflow_DTN104:
	PUSH	A	; beinhaltet EP Nummer
	PUSH	B
	PUSH	C

;   #################################
;   Start Servo0 und Servo1
;   #################################
	CPI		DEVFLAG,7
	BRNE	timer1_overflow_DTN104_not7

	LDI		A,2
	RCALL	getRXEP
	OUT		OCR1AH,B
	OUT		OCR1AL,C
	LDI		A,3
	RCALL	getRXEP
	OUT		OCR1BH,B
	OUT		OCR1BL,C
timer1_overflow_DTN104_not7:

;   #################################
;   Start Servo0 und Servo1
;   #################################
	CPI		DEVFLAG,0
	BRNE	timer1_overflow_DTN104_not0

	IN		i,PORTD
	ORI		i,0b11000000
	OUT		PORTD,i

	LDI		A,4
	RCALL	getRXEP
	OUT		OCR1AH,B
	OUT		OCR1AL,C
	LDI		A,5
	RCALL	getRXEP
	OUT		OCR1BH,B
	OUT		OCR1BL,C
timer1_overflow_DTN104_not0:

;   #################################
;   Start Servo2 und Servo3
;   #################################
	CPI		DEVFLAG,1
	BRNE	timer1_overflow_DTN104_not1

	IN		i,PORTD
	ORI		i,0b00110000
	OUT		PORTD,i

	LDI		A,6
	RCALL	getRXEP
	OUT		OCR1AH,B
	OUT		OCR1AL,C
	LDI		A,7
	RCALL	getRXEP
	OUT		OCR1BH,B
	OUT		OCR1BL,C
timer1_overflow_DTN104_not1:

;   #################################
;   Start Servo4 und Servo5
;   #################################
	CPI		DEVFLAG,2
	BRNE	timer1_overflow_DTN104_not2

	IN		i,PORTD
	ORI		i,0b00000011
	OUT		PORTD,i

	LDI		A,8
	RCALL	getRXEP
	OUT		OCR1AH,B
	OUT		OCR1AL,C
	LDI		A,9
	RCALL	getRXEP
	OUT		OCR1BH,B
	OUT		OCR1BL,C
timer1_overflow_DTN104_not2:

;   #################################
;   Start Servo6 und Servo7
;   #################################
	CPI		DEVFLAG,3
	BRNE	timer1_overflow_DTN104_not3

	IN		i,PORTB
	ORI		i,0b00001100
	OUT		PORTB,i

	LDI		A,10
	RCALL	getRXEP
	OUT		OCR1AH,B
	OUT		OCR1AL,C
	LDI		A,11
	RCALL	getRXEP
	OUT		OCR1BH,B
	OUT		OCR1BL,C
timer1_overflow_DTN104_not3:

;   #################################
;   Start Servo8 und Servo9
;   #################################
	cpi		DEVFLAG,4
	brne	timer1_overflow_DTN104_not4

	in		i,PORTB
	ori		i,0b00000001
	out		PORTB,i
	in		i,PORTA
	ori		i,0b00000001
	out		PORTA,i

	ldi		A,12
	rcall	getRXEP
	out		OCR1AH,B
	out		OCR1AL,C
	ldi		A,13
	rcall	getRXEP
	out		OCR1BH,B
	out		OCR1BL,C
timer1_overflow_DTN104_not4:

;   #################################
;   Start Servo10 und Servo11
;   #################################
	cpi		DEVFLAG,5
	brne	timer1_overflow_DTN104_not5

	in		i,PORTA
	ori		i,0b00000110
	out		PORTA,i
timer1_overflow_DTN104_not5:

	inc		DEVFLAG
	cpi		DEVFLAG,8
	brne	timer1_overflow_DTN104_not_reset
	ldi		DEVFLAG,0
timer1_overflow_DTN104_not_reset:

	POP		C
	POP		B 
	POP		A
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI


timer1_overflow_DTN106:
	PUSH	A	; beinhaltet EP Nummer
	PUSH	B
	PUSH	C

;   #################################
;   Prepare
;   #################################
	CPI		DEVFLAG,7
	BRNE	timer1_overflow_DTN106_not7

	LDI		A,2
	RCALL	getRXEP
	OUT		OCR1AH,B
	OUT		OCR1AL,C
timer1_overflow_DTN106_not7:

;   #################################
;   Start Servo1
;   #################################
	CPI		DEVFLAG,0
	BRNE	timer1_overflow_DTN106_not0

	IN		i,PORTD
	ORI		i,0b01000000
	OUT		PORTD,i

	LDI		A,3
	RCALL	getRXEP
	OUT		OCR1AH,B
	OUT		OCR1AL,C
timer1_overflow_DTN106_not0:

;   #################################
;   Start Servo2
;   #################################
	CPI		DEVFLAG,1
	BRNE	timer1_overflow_DTN106_not1

	IN		i,PORTD
	ORI		i,0b00100000
	OUT		PORTD,i

	LDI		A,4
	RCALL	getRXEP
	OUT		OCR1AH,B
	OUT		OCR1AL,C
timer1_overflow_DTN106_not1:

;   #################################
;   Start Servo3
;   #################################
	CPI		DEVFLAG,2
	BRNE	timer1_overflow_DTN106_not2

	IN		i,PORTD
	ORI		i,0b00010000
	OUT		PORTD,i

	LDI		A,5
	RCALL	getRXEP
	OUT		OCR1AH,B
	OUT		OCR1AL,C
timer1_overflow_DTN106_not2:

;   #################################
;   Start Servo4
;   #################################
	CPI		DEVFLAG,3
	BRNE	timer1_overflow_DTN106_not3

	IN		i,PORTD
	ORI		i,0b00000010
	OUT		PORTD,i
timer1_overflow_DTN106_not3:

	inc		DEVFLAG
	cpi		DEVFLAG,8
	brne	timer1_overflow_DTN106_not_reset
	ldi		DEVFLAG,0
timer1_overflow_DTN106_not_reset:

	POP		C
	POP		B 
	POP		A
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI

timer1_compare1a:
	push	temp
	in		temp,SREG
	PUSH	i	
	lds		i,DTN
	cpi		i,104
	brne	timer1_compare1a_not_DTN104
	rjmp	timer1_compare1a_DTN104
timer1_compare1a_not_DTN104:
	CPI		i,106
	BRNE	timer1_compare1a_not_DTN106
	RJMP	timer1_compare1a_DTN106
timer1_compare1a_not_DTN106:
	CPI		i,127
	BRNE	timer1_compare1a_not_DTN127
	RJMP	timer1_compare1a_DTN106
timer1_compare1a_not_DTN127:
	pop		i
	out		SREG,temp
	pop		temp
	reti

timer1_compare1a_DTN104:
	in		i,PORTD
	andi	i,~0b10100010 ;  Ausgnge 0 setzten
	out		PORTD,i
	in		i,PORTB
	andi	i,~0b00001001 ;  Ausgnge 0 setzten
	out		PORTB,i
	in		i,PORTA
	andi	i,~0b00000010 ;  Ausgnge 0 setzten
	out		PORTA,i
	POP		i
	out		SREG,temp
	pop		temp
	reti

timer1_compare1a_DTN106:
	in		i,PORTD
	andi	i,~0b01110010 ;  Ausgnge 0 setzten
	out		PORTD,i
	POP		i
	out		SREG,temp
	pop		temp
	reti

timer1_compare1b:
	push	temp
	in		temp,SREG
	PUSH	i	
	lds		i,DTN
	cpi		i,104
	brne	timer1_compare1b_not_DTN104
	rjmp	timer1_compare1b_DTN104
timer1_compare1b_not_DTN104:
	pop		i
	out		SREG,temp
	pop		temp
	reti

timer1_compare1b_DTN104:
	in		i,PORTD
	andi	i,~0b01010001 ;  Ausgnge 0 setzten
	out		PORTD,i
	in		i,PORTB
	andi	i,~0b00000100 ;  Ausgnge 0 setzten
	out		PORTB,i
	in		i,PORTA
	andi	i,~0b00000101 ;  Ausgnge 0 setzten
	out		PORTA,i

	POP		i
	out		SREG,temp
	pop		temp
	reti

timer1_overflow_DTN201:
	MOV		i,DEVDATX
	CPI		i,0
	BREQ	timer1_overflow_DTN201_zero
	DEC		i
	MOV		DEVDATX,i
timer1_overflow_DTN201_zero:
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI

;   #################################
;   TIMER2 Overflow Handler
;   #################################

timer2_overflow:
	SEI
	PUSH	temp
	IN		temp,SREG
	PUSH	i

	LDS		i,DTN
	CPI		i,120
	BRNE	timer2_overflow_not_DTN120
	RJMP	timer2_overflow_DTN120
timer2_overflow_not_DTN120:

	POP		i
	OUT		SREG,temp
	POP		temp
	RETI


timer2_overflow_DTN120:
; Timer wird neu geladen
	LDI		i,240
	OUT		TCNT2,i
	CPI		DEVFLAG2,0
	BREQ	timer2_overflow_DTN120_end
	DEC		DEVFLAG2

	IN		i,MCUCSR
	ANDI	i,~(1<<ISC2) ; INT2 aktiv auf fallende Flanke
	SBIS	P10_IN,P10_PIN
	ORI		i,(1<<ISC2) ; INT2 aktiv auf steigende Flanke
	OUT		MCUCSR,i

	ANDI	i,(1<<ISC2)
	CPI		i,0
	BRNE	timer2_overflow_DTN120_rx0

	MOV		i,DEVDATL

	CPI		DEVFLAG,5
	BRNE	timer2_overflow_DTN120_not_rxBIT5
	ORI		i,32
timer2_overflow_DTN120_not_rxBIT5:


	CPI		DEVFLAG,4
	BRNE	timer2_overflow_DTN120_not_rxBIT4
	ORI		i,16
timer2_overflow_DTN120_not_rxBIT4:

	CPI		DEVFLAG,3
	BRNE	timer2_overflow_DTN120_not_rxBIT3
	ORI		i,8
timer2_overflow_DTN120_not_rxBIT3:

	CPI		DEVFLAG,2
	BRNE	timer2_overflow_DTN120_not_rxBIT2
	ORI		i,4
timer2_overflow_DTN120_not_rxBIT2:

	CPI		DEVFLAG,1
	BRNE	timer2_overflow_DTN120_not_rxBIT1
	ORI		i,2
timer2_overflow_DTN120_not_rxBIT1:

	CPI		DEVFLAG,0
	BRNE	timer2_overflow_DTN120_not_rxBIT0
	ORI		i,1
timer2_overflow_DTN120_not_rxBIT0:

	MOV		DEVDATL,i
timer2_overflow_DTN120_rx0:

	CPI		DEVFLAG,0
	BRNE	timer2_overflow_DTN120_not_rxready
	LDI		A,2
	LDI		B,0
	MOV		C,DEVDATL
	RCALL	setTXEP

timer2_overflow_DTN120_not_rxready:

	IN		i,GIFR
	ORI		i,(1<<INTF2)	; INT2 Flag lschen
	OUT		GIFR,i

	IN		i,GICR
	ORI		i,(1<<INTF2)	; INT2 aktivieren
	OUT		GICR,i

	POP		i
	OUT		SREG,temp
	POP		temp
	RETI

timer2_overflow_DTN120_end:
	CBI		P9_OUT,P9_PIN
	CALL	DTN120_Reset
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI
