;#########################################################################
; BIOE Main-Programm for the ATMega16 Controller
; Copyright (C) 2008 Klaus Weichinger
;
; License: CC-GNU GPL
; http://creativecommons.org/licenses/GPL/2.0/
;#########################################################################
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor,
; Boston, MA  02110-1301, USA.
;#########################################################################

;  +---------------------------------+
;  |DTN                          AREF+-
;  |      IN          OUT        P15 +- ADC3-PA3
;  |EP2                          P14 +- ADC2-PA2
;  |EP3                          P13 +- ADC1-PA1
;  |EP4                          P12 +- ADC0-PA0
;  |EP5                          P11 +- PB0 (XCK/T0)
;  |EP6                          P10 +- PB2 (AIN0/INT2)
;  |EP7                          P9  +- PB3 (AIN1/OC0)
;  |EP8                          GND +-
;  |EP9                          P8  +- PB6 (MISO)
;  |EP10                         P7  +- PB5 (MOSI)
;  |EP11                         RES +-
;  |EP12                         P6  +- PB7 (SCK)
;  |EP13                         VCC +-
;  |EP14                         P5  +- PD0 (RXD)
;  |                             P4  +- PD1 (TXD)
;  |                             P3  +- PD4 (OC1B)
;  |                             P2  +- PD5 (OC1A)
;  |                             P1  +- PD6 (ICP1)
;  |                             P0  +- PD7 (OC2)
;  +---------------------------------+



.include "m16def.inc"

	.EQU	XTAL=16000000 ; Hz 

; Definition der Globalen-Register

	.DEF	ZERO		= r8	; Register beinhaltet eine 0
	.DEF	ONE			= r9	; Register beinhaltet eine 1
	.DEF	TXBUFF0		= r2
	.DEF	TXBUFF1		= r3
	.DEF	RXBUFF0		= r4
	.DEF	RXBUFF1		= r5
	
	.DEF	DEVDATX		= r6	; Dieses Byte steht fr das aktuelle Gert zur Verfgung
	.DEF	DEVDATY		= r7	; Dieses Byte steht fr das aktuelle Gert zur Verfgung
	.DEF	DEVDATL		= r0	; Dieses Byte steht fr das aktuelle Gert zur Verfgung
	.DEF	DEVDATH		= r1	; Dieses Byte steht fr das aktuelle Gert zur Verfgung

	.DEF	TXBUFFL		= r10
	.DEF	TXBUFFH 	= r11
	.DEF	RXBUFFL		= r12
	.DEF	RXBUFFH 	= r13
	.DEF	CHECKSUM	= r14
	.DEF	temp 		= r15


	.DEF	DEVFLAG2	= r16	; Dieses Byte steht fr das aktuelle Gert zur Verfgung
	.DEF	i 			= r17	; frei fr Benutzung
	.DEF	j 			= r18	; frei fr Benutzung
	.DEF	FPN1		= r17	; frei fr Benutzung
	.DEF	FPN2		= r18	; frei fr Benutzung
	.DEF	DEVFLAG		= r19	; Dieses Byte steht fr das aktuelle Gert zur Verfgung
	.DEF	A			= r20	; frei fr Benutzung
	.DEF	B			= r21	; frei fr Benutzung
	.DEF	C			= r22	; frei fr Benutzung
	.DEF	FPx			= r20	; frei fr Benutzung
	.DEF	FPy			= r21	; frei fr Benutzung
	.DEF	FPz			= r22	; frei fr Benutzung
	.DEF	FPA			= r6
	.DEF	FPB			= r7
	.DEF	FPC			= r18
	.DEF	BusState	= r23
	.DEF	RXEPFLAG	= r24
	.DEF	RXEPTEMP	= r25


; Definition der IO-Leitungen
	.EQU	TEST_DIR	= DDRB
	.EQU	TEST_IN		= PINB
	.EQU	TEST_OUT	= PORTB
	.EQU	TEST_PIN	= 1

	.EQU	LED_DIR		= DDRB
	.EQU	LED_IN		= PINB
	.EQU	LED_OUT		= PORTB
	.EQU	LED_PIN		= 4

	.EQU	USERLED_DIR	= DDRB
	.EQU	USERLED_IN	= PINB
	.EQU	USERLED_OUT	= PORTB
	.EQU	USERLED_PIN	= 1

	.EQU	CS_DIR		= DDRD
	.EQU	CS_IN		= PIND
	.EQU	CS_OUT		= PORTD
	.EQU	CS_PIN		= 2

	.EQU	CLK_DIR		= DDRD
	.EQU	CLK_IN		= PIND
	.EQU	CLK_OUT		= PORTD
	.EQU	CLK_PIN		= 3

	.EQU	DO_DIR		= DDRC
	.EQU	DO_IN		= PINC
	.EQU	DO_OUT		= PORTC
	.EQU	DO_PINS		= $0F

	.EQU	DO0_DIR		= DDRC
	.EQU	DO0_IN		= PINC
	.EQU	DO0_OUT		= PORTC
	.EQU	DO0_PIN		= 0

	.EQU	DO1_DIR		= DDRC
	.EQU	DO1_IN		= PINC
	.EQU	DO1_OUT		= PORTC
	.EQU	DO1_PIN		= 1

	.EQU	DO2_DIR		= DDRC
	.EQU	DO2_IN		= PINC
	.EQU	DO2_OUT		= PORTC
	.EQU	DO2_PIN		= 2

	.EQU	DO3_DIR		= DDRC
	.EQU	DO3_IN		= PINC
	.EQU	DO3_OUT		= PORTC
	.EQU	DO3_PIN		= 3

	.EQU	DI_DIR		= DDRC
	.EQU	DI_IN		= PINC
	.EQU	DI_OUT		= PORTC
	.EQU	DI_PINS		= $F0

	.EQU	DI0_DIR		= DDRC
	.EQU	DI0_IN		= PINC
	.EQU	DI0_OUT		= PORTC
	.EQU	DI0_PIN		= 4

	.EQU	DI1_DIR		= DDRC
	.EQU	DI1_IN		= PINC
	.EQU	DI1_OUT		= PORTC
	.EQU	DI1_PIN		= 5

	.EQU	DI2_DIR		= DDRC
	.EQU	DI2_IN		= PINC
	.EQU	DI2_OUT		= PORTC
	.EQU	DI2_PIN		= 6

	.EQU	DI3_DIR		= DDRC
	.EQU	DI3_IN		= PINC
	.EQU	DI3_OUT		= PORTC
	.EQU	DI3_PIN		= 7

	.EQU	ADR_DIR		= DDRA
	.EQU	ADR_IN		= PINA
	.EQU	ADR_OUT		= PORTA
	.EQU	ADR_PINS	= $F0

	.EQU	P0_DIR		= DDRD
	.EQU	P0_IN		= PIND
	.EQU	P0_OUT		= PORTD
	.EQU	P0_PIN		= 7

	.EQU	P1_DIR		= DDRD
	.EQU	P1_IN		= PIND
	.EQU	P1_OUT		= PORTD
	.EQU	P1_PIN		= 6

	.EQU	P2_DIR		= DDRD
	.EQU	P2_IN		= PIND
	.EQU	P2_OUT		= PORTD
	.EQU	P2_PIN		= 5

	.EQU	P3_DIR		= DDRD
	.EQU	P3_IN		= PIND
	.EQU	P3_OUT		= PORTD
	.EQU	P3_PIN		= 4

	.EQU	P4_DIR		= DDRD
	.EQU	P4_IN		= PIND
	.EQU	P4_OUT		= PORTD
	.EQU	P4_PIN		= 1

	.EQU	P5_DIR		= DDRD
	.EQU	P5_IN		= PIND
	.EQU	P5_OUT		= PORTD
	.EQU	P5_PIN		= 0

	.EQU	P6_DIR		= DDRB
	.EQU	P6_IN		= PINB
	.EQU	P6_OUT		= PORTB
	.EQU	P6_PIN		= 7

	.EQU	P7_DIR		= DDRB
	.EQU	P7_IN		= PINB
	.EQU	P7_OUT		= PORTB
	.EQU	P7_PIN		= 5

	.EQU	P8_DIR		= DDRB
	.EQU	P8_IN		= PINB
	.EQU	P8_OUT		= PORTB
	.EQU	P8_PIN		= 6

	.EQU	P9_DIR		= DDRB
	.EQU	P9_IN		= PINB
	.EQU	P9_OUT		= PORTB
	.EQU	P9_PIN		= 3

	.EQU	P10_DIR		= DDRB
	.EQU	P10_IN		= PINB
	.EQU	P10_OUT		= PORTB
	.EQU	P10_PIN		= 2

	.EQU	P11_DIR		= DDRB
	.EQU	P11_IN		= PINB
	.EQU	P11_OUT		= PORTB
	.EQU	P11_PIN		= 0

	.EQU	P12_DIR		= DDRA
	.EQU	P12_IN		= PINA
	.EQU	P12_OUT		= PORTA
	.EQU	P12_PIN		= 0

	.EQU	P13_DIR		= DDRA
	.EQU	P13_IN		= PINA
	.EQU	P13_OUT		= PORTA
	.EQU	P13_PIN		= 1

	.EQU	P14_DIR		= DDRA
	.EQU	P14_IN		= PINA
	.EQU	P14_OUT		= PORTA
	.EQU	P14_PIN		= 2

	.EQU	P15_DIR		= DDRA
	.EQU	P15_IN		= PINA
	.EQU	P15_OUT		= PORTA
	.EQU	P15_PIN		= 3

.MACRO FP_LOADCONST ;(Adresse)
        LDI     ZL, LOW(@0<<1)
        LDI     ZH, HIGH(@0<<1)
		CALL	FP_LOADFLASH
.ENDMACRO


.ORG 0x000
	RJMP init				; Reset Handler
.ORG INT0addr
	RJMP int0_handler		; IRQ0 Handler
.ORG INT1addr
	RJMP int1_handler		; IRQ1 Handler
.ORG OC1Aaddr
	RJMP timer1_compare1a	; Output Compare1A Handler
.ORG OC1Baddr
	RJMP timer1_compare1b	; Output Compare1B Handler
.ORG OVF1addr
	RJMP timer1_overflow	; Timer1 Overflow Handler
.ORG OVF2addr
	RJMP timer2_overflow	; Timer2 Overflow Handler
.ORG INT2addr
	RJMP int2_handler		; IRQ2 Handler

;########################################
; Initialisierung und MAIN-Programm
;########################################
init:
; Initialisierung des STACK's

	LDI		i, LOW(RAMEND)
	OUT		SPL, i
	LDI		i, HIGH(RAMEND)
	OUT		SPH, i

	LDI		i,0
	MOV		ZERO,i

	LDI		i,1
	MOV		ONE,i

	SBI		LED_DIR, LED_PIN ; LED, Ausgang
	CBI		LED_OUT, LED_PIN ; LED, Aus
	SBI		USERLED_DIR, USERLED_PIN ; USERLED, Ausgang
	CBI		USERLED_OUT, USERLED_PIN ; LED, Aus

	SBI		TEST_DIR, TEST_PIN ; TEST-PIN, Ausgang
	CBI		TEST_OUT, TEST_PIN ; TEST-PIN, Ausgang	

; ---------------------------------------
; Initialisierung des 4-Bit Parallel-Bus
; ---------------------------------------
; CS und CLK als Eingnge definieren
	CBI		CS_DIR, CS_PIN
	CBI		CLK_DIR, CLK_PIN
; DO hochohmig machen
	IN		i,DO_DIR
	ANDI	i,~DO_PINS
	OUT		DO_DIR,i
; DI hochohmig machen
	IN		i,DI_DIR
	ANDI	i,~DI_PINS
	OUT		DI_DIR,i
; ADR hochohmig machen
	IN		i,ADR_DIR
	ANDI	i,~ADR_PINS
	OUT		ADR_DIR,i
; CS und CLK Pullups ein
	SBI		CS_OUT, CS_PIN
	SBI		CLK_OUT, CLK_PIN
; DI Pullups ein
	IN		i,DI_OUT
	ANDI	i,~DI_PINS
	OUT		DI_OUT,i
; DO Pullups ein
	IN		i,DO_OUT
	ANDI	i,~DO_PINS
	OUT		DO_OUT,i
; ADR Pullups ein
	IN		i,ADR_OUT
	ORI		i,ADR_PINS
	OUT		ADR_OUT,i
; Interrupts fr CS und CLK einrichten
	LDI		i,0b00001101 ; CS bei jeder Flanke, CLK bei steigender Flanke
	OUT		MCUCR,i

	IN		i,GIFR
	ORI		i,(1<<INTF1)+(1<<INTF0)			; INT1 und INT0 Flag lschen
	OUT		GIFR,i

    LDI		i, 0b01000000  ; INT0 - CS aktivieren
	OUT		GICR, i


	LDI		i,HIGH(TXEP)
	MOV		TXBUFFH,i
	LDI		i,LOW(TXEP)
	MOV		TXBUFFL,i
	LDI		i,HIGH(RXEP)
	MOV		RXBUFFH,i
	LDI		i,LOW(RXEP)
	MOV		RXBUFFL,i


	LDI		A,14
	LDI		B,15
	LDI		C,15
	RCALL	setTXEP

	LDI		BusState,$00	; Buszustand initialisieren

	LDI		i,0
	STS		DTN,i
	RCALL	initDeviceType
; ---------------------------------------
; Initialisierung Allgemein
; ---------------------------------------

	SEI	; Alle Interrupts aktivieren

	LDI		RXEPFLAG,$FF
main:	
	CPI		RXEPFLAG,0
	BRNE	main_not_EP0
	LDI		RXEPFLAG,$FF	; FLAG lschen
	RCALL	process_EP0
main_not_EP0:

	CPI		RXEPFLAG,15
	BRNE	main_not_EP15
	; Endpoint Test
	LDI		RXEPFLAG,$FF	; FLAG lschen
	LDI		A,15
	RCALL	getRXEP
	RCALL	setTXEP
main_not_EP15:


;  Gertetypen behandeln
	LDI		ZL,LOW(doDTN)
	LDI		ZH,HIGH(doDTN)
	LDS		i,DTN
	ADD		ZL,i
	ADC		ZH,ZERO
	LSL		ZL
	ROL		ZH
	LPM		i,Z+
	LPM		ZH,Z
	MOV		ZL,i
	LDI		i,0	; damit process_DTN ausgefhrt wird
	IJMP
;	RJMP	process_DTN102
processDeviceType_end:	

	RJMP	main			; Endlosschleife des Hauptprogrammes


;########################################
; Interrupt Sevice Routinen
;########################################
int0_handler: ; CS-Interrupt
	PUSH	temp
	IN		temp,SREG
	PUSH	i

; Hochohmig machen
	IN		i,DI_DIR
	ANDI	i,~DI_PINS
	OUT		DI_DIR,i

; LED ausschalten
	CBI		LED_OUT,LED_PIN

; CS-Level einlesen und auswerten
	SBIC	CS_IN,CS_PIN
	RJMP	int0_handler_CS_HIGH

int0_handler_CS_LOW:
	CBI		LED_OUT,LED_PIN


	IN		i,GICR
	ANDI	i, ~0b10000000  ; INT1 - CLK deaktivieren
	OUT		GICR, i
		
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI

int0_handler_CS_HIGH:
	LDI		BusState,1

	IN		i,GIFR
	ORI		i,(1<<INTF1)			; INT1 Flag lschen
	OUT		GIFR,i

	IN		i,GICR
	ORI		i, 0b10000000  ; INT1 - CLK aktivieren
	OUT		GICR, i

	POP		i
	OUT		SREG,temp
	POP		temp
	RETI


;-----------------------------------------------------------
int1_handler_ADR:
	PUSH	j

	IN		j,ADR_IN
	COM		j
	ANDI	j,ADR_PINS
	SWAP	j

	IN		i,DO_IN
	ANDI	i,DO_PINS

	MOV		CHECKSUM,i

	CP		i,j
	BRNE	int1_handler_ADR_NO
; Adresse stimmt mit Gert berein
	SBI		LED_OUT,LED_PIN
	POP		j

	INC		BusState
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI
;	RJMP	int1_handler_ENDE

int1_handler_ADR_NO:
	IN		i,GICR
    ANDI	i, ~0b10000000  ; INT1 - CLK deaktivieren
	OUT		GICR, i
	LDI		BusState,$FE
	POP		j

	INC		BusState
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI
;	RJMP	int1_handler_ENDE

;-----------------------------------------------------------
int1_handler_EP:
	; DI als Ausgang Schalten
	IN		i,DI_DIR
	ORI		i,DI_PINS
	OUT		DI_DIR,i

	; EP von DO einlesen
	IN		i,DO_IN
	ANDI	i,DO_PINS
	EOR		CHECKSUM,i
	ORI		i,$F0
	MOV		RXEPTEMP,i
	ANDI	i,DO_PINS
	ROL		i

	; Zeiger fr die Endpoints initialisieren
	MOVW	XL:XH,TXBUFFL:TXBUFFH
	MOVW	YL:YH,RXBUFFL:RXBUFFH

	ADD		XL,i
	ADC		XH,zero
	ADD		YL,i
	ADC		YH,zero

	LD		TXBUFF1,X+
	LD		TXBUFF0,X+

	INC		BusState
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI
;	RJMP	int1_handler_ENDE

int1_handler_D3:
	MOV		i,TXBUFF0
	ANDI	i,$F0
	OUT		DI_OUT,i
	SWAP	i
	EOR		CHECKSUM,i

	IN		i,DO_IN
	ANDI	i,DO_PINS
	EOR		CHECKSUM,i
	SWAP	i
	MOV		RXBUFF0,i

	INC		BusState
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI
;	rjmp	int1_handler_ENDE

int1_handler: ; CLK-Interrupt
	PUSH	temp
	IN		temp,SREG
	PUSH	i

	CPI		BusState,4
	BRGE	int1_handler_4567
int1_handler_123:
	CPI		BusState,3
	BRGE	int1_handler_D3
int1_handler_12:
	CPI		BusState,2
	BRGE	int1_handler_EP
	RJMP	int1_handler_ADR

int1_handler_4567:
	CPI		BusState,6
	BRGE	int1_handler_67
int1_handler_45:
	CPI		BusState,5
	BRGE	int1_handler_D1
	RJMP	int1_handler_D2

int1_handler_67:
	CPI		BusState,7
	BRGE	int1_handler_CHECKSUM
	RJMP	int1_handler_D0

	INC		BusState
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI
;	rjmp	int1_handler_ENDE

;-----------------------------------------------------------


int1_handler_D2:
	MOV		i,TXBUFF0
	ANDI	i,$0F
	EOR		CHECKSUM,i
	SWAP	i
	OUT		DI_OUT,i

	IN		i,DO_IN
	ANDI	i,DO_PINS
	EOR		CHECKSUM,i
	ADD		RXBUFF0,i

	INC		BusState
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI
;	rjmp	int1_handler_ENDE


int1_handler_D1:
	MOV		i,TXBUFF1
	ANDI	i,$F0
	OUT		DI_OUT,i
	SWAP	i
	EOR		CHECKSUM,i

	IN		i,DO_IN
	ANDI	i,DO_PINS
	EOR		CHECKSUM,i
	SWAP	i
	MOV		RXBUFF1,i

	INC		BusState
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI
;	rjmp	int1_handler_ENDE

int1_handler_D0:
	MOV		i,TXBUFF1
	ANDI	i,$0F
	EOR		CHECKSUM,i
	SWAP	i
	OUT		DI_OUT,i

	IN		i,DO_IN
	ANDI	i,DO_PINS
	EOR		CHECKSUM,i
	ADD		RXBUFF1,i

	INC		BusState
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI
;	rjmp	int1_handler_ENDE

int1_handler_CHECKSUM:	

	SWAP	CHECKSUM
	OUT		DI_OUT,CHECKSUM
	SWAP	CHECKSUM


	IN		i,DO_IN
	ANDI	i,DO_PINS
	CP		i,CHECKSUM
	BRNE	int1_handler_CHECKSUM_notvalid
	ST		Y+,RXBUFF1
	ST		Y+,RXBUFF0
	ANDI	RXEPTEMP,$0F
	CPI		RXEPTEMP,1 // EP1 wurde gelesen, dann kein RXFLAG setzen
	BREQ	init_handler_CHECKSUM_EP1RXignore
	MOV		RXEPFLAG,RXEPTEMP
init_handler_CHECKSUM_EP1RXignore:
	CBI		LED_OUT,LED_PIN
int1_handler_CHECKSUM_notvalid:

	INC		BusState
	POP		i
	OUT		SREG,temp
	POP		temp
	RETI
;	rjmp	int1_handler_ENDE

int1_handler_ENDE:
	INC		BusState

	POP		i
	OUT		SREG,temp
	POP		temp
	RETI

.include	"DTNInterrupts.asm"

;########################################
; Unterprogramme
;########################################
process_EP0:
	PUSH	temp
	IN		temp,SREG

	PUSH	A
	PUSH	B
	PUSH	C
	PUSH	i

	LDI		A,0
	RCALL	getRXEP

	;  DTN-Nummer setzen
	CPI		B,$00
	BRNE	process_EP0_not_00
	STS		DTN,C
	RCALL	initDeviceType
	LDI		A,1
	LDI		B,0
	LDS		C,DTN
	RCALL	setTXEP
process_EP0_not_00:

	;  DTN-Nummer auslesen
	CPI		B,$01
	BRNE	process_EP0_not_01
	LDI		A,1
	LDI		B,1
	LDS		C,DTN
	RCALL	setTXEP
process_EP0_not_01:

	;  Settings Done
	CPI		B,$02
	BRNE	process_EP0_not_02
	STS		SDONE,ONE
	LDI		A,1
	LDI		B,$02
	LDS		C,SDONE
	RCALL	setTXEP
process_EP0_not_02:

;   Die 8 Settings-Bytes werden geschrieben
	CPI		B,$10
	BRNE	process_EP0_not_10
	STS		S0,C
	LDI		A,1
	LDI		B,$10
	LDS		C,S0
	RCALL	setTXEP
process_EP0_not_10:

	CPI		B,$11
	BRNE	process_EP0_not_11
	STS		S1,C
	LDI		A,1
	LDI		B,$11
	LDS		C,S1
	RCALL	setTXEP
process_EP0_not_11:

	CPI		B,$12
	BRNE	process_EP0_not_12
	STS		S2,C
	LDI		A,1
	LDI		B,$12
	LDS		C,S2
	RCALL	setTXEP
process_EP0_not_12:

	CPI		B,$13
	BRNE	process_EP0_not_13
	STS		S3,C
	LDI		A,1
	LDI		B,$13
	LDS		C,S3
	RCALL	setTXEP
process_EP0_not_13:

	CPI		B,$14
	BRNE	process_EP0_not_14
	STS		S4,C
	LDI		A,1
	LDI		B,$14
	LDS		C,S4
	RCALL	setTXEP
process_EP0_not_14:

	CPI		B,$15
	BRNE	process_EP0_not_15
	STS		S5,C
	LDI		A,1
	LDI		B,$15
	LDS		C,S5
	RCALL	setTXEP
process_EP0_not_15:

	CPI		B,$16
	BRNE	process_EP0_not_16
	STS		S6,C
	LDI		A,1
	LDI		B,$16
	LDS		C,S6
	RCALL	setTXEP
process_EP0_not_16:

	CPI		B,$17
	BRNE	process_EP0_not_17
	STS		S7,C
	LDI		A,1
	LDI		B,$17
	LDS		C,S7
	RCALL	setTXEP
process_EP0_not_17:


;   Die 8 Settings-Bytes werden gelesen
	CPI		B,$20
	BRNE	process_EP0_not_20
	LDI		A,1
	LDI		B,$20
	LDS		C,S0
	RCALL	setTXEP
process_EP0_not_20:

	CPI		B,$21
	BRNE	process_EP0_not_21
	LDI		A,1
	LDI		B,$21
	LDS		C,S1
	RCALL	setTXEP
process_EP0_not_21:

	CPI		B,$22
	BRNE	process_EP0_not_22
	LDI		A,1
	LDI		B,$22
	LDS		C,S2
	RCALL	setTXEP
process_EP0_not_22:

	CPI		B,$23
	BRNE	process_EP0_not_23
	LDI		A,1
	LDI		B,$23
	LDS		C,S3
	RCALL	setTXEP
process_EP0_not_23:

	CPI		B,$24
	BRNE	process_EP0_not_24
	LDI		A,1
	LDI		B,$24
	LDS		C,S4
	RCALL	setTXEP
process_EP0_not_24:

	CPI		B,$25
	BRNE	process_EP0_not_25
	LDI		A,1
	LDI		B,$25
	LDS		C,S5
	RCALL	setTXEP
process_EP0_not_25:

	CPI		B,$26
	BRNE	process_EP0_not_26
	LDI		A,1
	LDI		B,$26
	LDS		C,S6
	RCALL	setTXEP
process_EP0_not_26:

	CPI		B,$27
	BRNE	process_EP0_not_27
	LDI		A,1
	LDI		B,$27
	LDS		C,S7
	RCALL	setTXEP
process_EP0_not_27:

	;  Dummy-Befehl, damit berprft werden kann,
	;  ob der neue Befehl angekommen ist.
	CPI		B,$FF
	BRNE	process_EP0_not_FF
	LDI		A,1
	RCALL	setTXEP
process_EP0_not_FF:

	POP		i
	POP		C
	POP		B
	POP		A
	OUT		SREG,temp
	POP		temp

	RET


setTXEP:
	PUSH	temp
	IN		temp,SREG
	PUSH	A	; beinhaltet EP Nummer

	PUSH	ZL
	PUSH	ZH

	MOVW	ZL:ZH,TXBUFFL:TXBUFFH
	CLC
	ROL		A
	ADD		ZL,A
	ADC		ZH,zero

	CLI		; Alle Interrupts deaktivieren	
	ST		Z+,C
	ST		Z+,B
	SEI		; Alle Interrupts aktivieren

	POP		ZH
	POP		ZL
		
	POP		A
	OUT		SREG,temp
	POP		temp
	RET

getRXEP:
	PUSH	temp
	IN		temp,SREG
	PUSH	A ; beinhaltet EP Nummer
	; liefert B=HIGH Byte, C=LOW-Byte

	PUSH	ZL
	PUSH	ZH
	MOVW	ZL:ZH,RXBUFFL:RXBUFFH
	CLC
	ROL		A
	ADD		ZL,A
	ADC		ZH,zero
	

	CLI		; Alle Interrupts deaktivieren	
	LD		C,Z+
	LD		B,Z+
	SEI		; Alle Interrupts aktivieren

	POP		ZH
	POP		ZL

	POP		A
	OUT		SREG,temp
	POP		temp
	RET

setDefaultRXEP:
	PUSH	temp
	IN		temp,SREG
	PUSH	A ; beinhaltet EP Nummer
	; liefert B=HIGH Byte, C=LOW-Byte

	PUSH	ZL
	PUSH	ZH
	MOVW	ZL:ZH,RXBUFFL:RXBUFFH
	CLC
	ROL		A
	ADD		ZL,A
	ADC		ZH,zero
	

	ST		Z+,C
	ST		Z+,B

	POP		ZH
	POP		ZL

	POP		A
	OUT		SREG,temp
	POP		temp
	RET

initDeviceType:
	PUSH	temp
	IN		temp,SREG
	PUSH	i
	
; Alles Zurcksetzen
	LDI		i,$00
	OUT		TIMSK,i	; Deaktivieren aller Timer Interruptflags

	LDI		i,$FF
	OUT		TIFR,i	; lschen aller Flags (schreiben einer 1 lscht)

; Timer0 Register zurcksetzen
	LDI		i,$00
	OUT		TCCR0,i
	OUT		TCNT0,i
	OUT		OCR0,i
	
; Timer1 Register zurcksetzen
	LDI		i,$00
	OUT		TCCR1A,i
	OUT		TCCR1B,i

	OUT		TCNT1H,i
	OUT		TCNT1L,i

	OUT		OCR1AH,i
	OUT		OCR1AL,i

	OUT		OCR1BH,i
	OUT		OCR1BL,i

	OUT		ICR1H,i
	OUT		ICR1L,i

; Timer2 Register zurcksetzen
	LDI		i,$00
	OUT		TCCR2,i
	OUT		TCNT2,i
	OUT		OCR2,i

; INT2 deaktivieren
	IN		i,GICR
    ANDI	i,~(1<<INT2)	; INT2 Deaktivieren
	OUT		GICR, i

	IN		i,GIFR
	ORI		i,(1<<INTF2)	; INT2 Flag lschen
	OUT		GIFR,i

	

; ADC Register zurcksetzen
	OUT		ADMUX,zero
	OUT		ADCSRA,zero

; UART Register zurcksetzen
	LDI		i,0b00100000
	OUT		UCSRA,i
	OUT 	UCSRB,zero
	LDI		i,0b10000110
	OUT		UCSRC,i
	OUT		UBRRH,zero
	OUT		UBRRL,zero

; Led zurcksetzen
	CBI		USERLED_OUT, USERLED_PIN ; LED, Aus

; Ports zurcksetzen (Pullups aus, alle Eingnge)
	IN		i,PORTB
	ANDI	i,~0b11101101
	OUT		PORTB,i

	IN		i,PORTD
	ANDI	i,~0b11110011
	OUT		PORTD,i

	IN		i,PORTA
	ANDI	i,~0b00001111
	OUT		PORTA,i

	IN		i,DDRB
	ANDI	i,~0b11101101
	OUT		DDRB,i

	IN		i,DDRD
	ANDI	i,~0b11110011
	OUT		DDRD,i

	in		i,DDRA
	andi	i,~0b00001111
	out		DDRA,i
; RXEP und TXEP Defaultwerte setzen fr EP2..EP14
	LDI		B,0
	LDI		C,0

	LDI		A,2
initDeviceType_InitEP:
	RCALL	setTXEP
	RCALL	setDefaultRXEP
	INC		A
	CPI		A,15
	BRNE	initDeviceType_InitEP

; Settings Bytes S0...S7 initialisieren
	STS		S0,ZERO
	STS		S1,ZERO
	STS		S2,ZERO
	STS		S3,ZERO
	STS		S4,ZERO
	STS		S5,ZERO
	STS		S6,ZERO
	STS		S7,ZERO
	STS		SDONE,ZERO


; Initialisierung des Gertes

	LDI		ZL,LOW(doDTN)
	LDI		ZH,HIGH(doDTN)
	LDS		i,DTN
	ADD		ZL,i
	ADC		ZH,ZERO
	LSL		ZL
	ROL		ZH
	LPM		i,Z+
	LPM		ZH,Z
	MOV		ZL,i
	LDI		i,1	; damit init_DTN ausgefhrt wird
	IJMP
initDeviceType_end:
	POP		i
	OUT		SREG,temp
	POP		temp
	RET

;============================================
; Aktivierungsliste der DTN-Konfigurationen
;============================================

.include "bioedtnjumplist.asm"

;============================================
; Default DTN 000 (keine Aktion)
;============================================


do_DTN000:
	CPI		i,0
	BRNE	init_DTN000
	RJMP	process_DTN000
init_DTN000:
	PUSH	temp
	IN		temp,SREG
	PUSH	i
	MOV		DEVDATL,ZERO
	MOV		DEVDATH,ZERO
	POP		i
	OUT		SREG,temp
	POP		temp
	JMP	initDeviceType_end

process_DTN000:
	ADD		DEVDATL,ONE
	ADC		DEVDATH,ZERO
	MOV		i,DEVDATH
	LSL		i
	BRCS	process_DTN000_LEDON
	CBI		USERLED_OUT, USERLED_PIN ; LED, Aus
	JMP	processDeviceType_end
process_DTN000_LEDON:
	SBI		USERLED_OUT, USERLED_PIN ; LED, An
	JMP	processDeviceType_end



;============================================
; Module werden geladen
;============================================
.include "bioedtnlist.asm"

;============================================
; Fixed Point Arithmetic
;============================================
FP_SETFPN1_PTR:
	LDI		ZL,LOW(FPNUMBERS)
	LDI		ZH,HIGH(FPNUMBERS)
	MOV		i,FPN1
	JMP		FP_SETCONTINUE_PTR
FP_SETFPN2_PTR:
	LDI		ZL,LOW(FPNUMBERS)
	LDI		ZH,HIGH(FPNUMBERS)
	MOV		i,FPN2
FP_SETCONTINUE_PTR:
	ADD		ZL,i
	ADC		ZH,ZERO
	ADD		ZL,i
	ADC		ZH,ZERO
	ADD		ZL,i
	ADC		ZH,ZERO
	ADD		ZL,i
	ADC		ZH,ZERO
	ADD		ZL,i
	ADC		ZH,ZERO
	ADD		ZL,i
	ADC		ZH,ZERO
	ADD		ZL,i
	ADC		ZH,ZERO
	ADD		ZL,i
	ADC		ZH,ZERO
	RET

FP_LOAD:
; Ldt FPN1 in den ACCUMULATOR
; 83 cycles, 5.19us
	PUSH	temp
	IN		temp,SREG
	PUSH	FPN1
	PUSH	i
	RCALL	FP_SETFPN1_PTR

	LD		i,Z+
	STS		FPACC0,i
	LD		i,Z+
	STS		FPACC1,i
	LD		i,Z+
	STS		FPACC2,i
	LD		i,Z+
	STS		FPACC3,i
	LD		i,Z+
	STS		FPACC4,i
	LD		i,Z+
	STS		FPACC5,i
	LD		i,Z+
	STS		FPACC6,i
	LD		i,Z+
	STS		FPACC7,i

	POP		i
	POP		FPN1
	OUT		SREG,temp
	POP		temp
	RET

FP_LOADFLASH:
; Ldt die auf Z zeigende Zahl in den ACCUMULATOR
; 83 cycles, 5.19us
	PUSH	temp
	IN		temp,SREG
	PUSH	i

	LPM		i,Z+
	STS		FPACC7,i
	LPM		i,Z+
	STS		FPACC6,i
	LPM		i,Z+
	STS		FPACC5,i
	LPM		i,Z+
	STS		FPACC4,i
	LPM		i,Z+
	STS		FPACC3,i
	LPM		i,Z+
	STS		FPACC2,i
	LPM		i,Z+
	STS		FPACC1,i
	LPM		i,Z+
	STS		FPACC0,i

	POP		i
	OUT		SREG,temp
	POP		temp
	RET

FP_COPYACC2B:
  PUSH  i
  LDS   i,FPACC0
  STS   FPB0,i
  LDS   i,FPACC1
  STS   FPB1,i
  LDS   i,FPACC2
  STS   FPB2,i
  LDS   i,FPACC3
  STS   FPB3,i
  LDS   i,FPACC4
  STS   FPB4,i
  LDS   i,FPACC5
  STS   FPB5,i
  LDS   i,FPACC6
  STS   FPB6,i
  LDS   i,FPACC7
  STS   FPB7,i
  POP   i
  RET

FP_COPYB2ACC:
  PUSH  i
  LDS   i,FPB0
  STS   FPACC0,i
  LDS   i,FPB1
  STS   FPACC1,i
  LDS   i,FPB2
  STS   FPACC2,i
  LDS   i,FPB3
  STS   FPACC3,i
  LDS   i,FPB4
  STS   FPACC4,i
  LDS   i,FPB5
  STS   FPACC5,i
  LDS   i,FPB6
  STS   FPACC6,i
  LDS   i,FPB7
  STS   FPACC7,i
  POP   i
  RET

FP_LOADB:
; Ldt FPN1 in den ACCUMULATOR
; 83 cycles, 5.19us
	PUSH	temp
	IN		temp,SREG
	PUSH	FPN1
	PUSH	i
	RCALL	FP_SETFPN1_PTR

	LD		i,Z+
	STS		FPB0,i
	LD		i,Z+
	STS		FPB1,i
	LD		i,Z+
	STS		FPB2,i
	LD		i,Z+
	STS		FPB3,i
	LD		i,Z+
	STS		FPB4,i
	LD		i,Z+
	STS		FPB5,i
	LD		i,Z+
	STS		FPB6,i
	LD		i,Z+
	STS		FPB7,i

	POP		i
	POP		FPN1
	OUT		SREG,temp
	POP		temp
	RET

FP_SAVE:
; Speichert den ACCUMULATOR in FPN1
; 84 cycles, 5.25us
; Fertig
	PUSH	temp
	IN		temp,SREG
	PUSH	FPN1
	PUSH	i
	CALL	FP_SETFPN1_PTR

	LDS		i,FPACC0
	ST		Z+,i
	LDS		i,FPACC1
	ST		Z+,i
	LDS		i,FPACC2
	ST		Z+,i
	LDS		i,FPACC3
	ST		Z+,i
	LDS		i,FPACC4
	ST		Z+,i
	LDS		i,FPACC5
	ST		Z+,i
	LDS		i,FPACC6
	ST		Z+,i
	LDS		i,FPACC7
	ST		Z+,i

	POP		i
	POP		FPN1
	OUT		SREG,temp
	POP		temp
	RET

FP_NEG:
; AKKUMULATOR wird *(-1) berechnet
; 71 cycles, 4.44us
; Fertig
	PUSH	temp
	IN		temp,SREG
	PUSH	FPx
	PUSH	FPy
	LDI		FPy,$FF

	LDS		FPx,FPACC0
	EOR		FPx,FPy
	ADD		FPx,ONE
	STS		FPACC0,FPx

	LDS		FPx,FPACC1
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPACC1,FPx

	LDS		FPx,FPACC2
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPACC2,FPx

	LDS		FPx,FPACC3
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPACC3,FPx

	LDS		FPx,FPACC4
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPACC4,FPx

	LDS		FPx,FPACC5
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPACC5,FPx

	LDS		FPx,FPACC6
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPACC6,FPx

	LDS		FPx,FPACC7
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPACC7,FPx

	POP		FPy
	POP		FPx
	OUT		SREG,temp
	POP		temp
	RET

FP_NEG_B:
; Register B wird *(-1) berechnet
; Fertig
	PUSH	temp
	IN		temp,SREG
	PUSH	FPx
	PUSH	FPy
	LDI		FPy,$FF

	LDS		FPx,FPB0
	EOR		FPx,FPy
	ADD		FPx,ONE
	STS		FPB0,FPx

	LDS		FPx,FPB1
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPB1,FPx

	LDS		FPx,FPB2
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPB2,FPx

	LDS		FPx,FPB3
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPB3,FPx

	LDS		FPx,FPB4
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPB4,FPx

	LDS		FPx,FPB5
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPB5,FPx

	LDS		FPx,FPB6
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPB6,FPx

	LDS		FPx,FPB7
	EOR		FPx,FPy
	ADC		FPx,ZERO
	STS		FPB7,FPx

	POP		FPy
	POP		FPx
	OUT		SREG,temp
	POP		temp
	RET

FP_ROUND2INT:
; Rundet zur nchsten ganzzahligen Integer Zahl
	PUSH	temp
	IN		temp,SREG
	PUSH	FPx
	PUSH	FPy

	STS		FPACC0,ZERO

	STS		FPACC1,ZERO

	STS		FPACC2,ZERO

	LDI   FPy,128
	LDS		FPx,FPACC3
	ADD		FPx,FPy
	STS		FPACC3,FPx

	LDS		FPx,FPACC4
	ADC		FPx,ZERO
	STS		FPACC4,FPx

	LDS		FPx,FPACC5
	ADC		FPx,ZERO
	STS		FPACC5,FPx

	LDS		FPx,FPACC6
	ADC		FPx,ZERO
	STS		FPACC6,FPx

	LDS		FPx,FPACC7
	ADC		FPx,ZERO
	STS		FPACC7,FPx

	POP		FPy
	POP		FPx
	OUT		SREG,temp
	POP		temp
	RET

FP_ADD:
; Addiert FPN1 zum AKKUMULATOR
; 112 cycles, 7us
; Fertig
	PUSH	temp
	IN		temp,SREG
	PUSH	FPN1
	PUSH	FPx
	PUSH	FPy
	CALL	FP_SETFPN1_PTR

	LD		FPy,Z+
	LDS		FPx,FPACC0
	ADD		FPx,FPy
	STS		FPACC0,FPx

	LD		FPy,Z+
	LDS		FPx,FPACC1
	ADC		FPx,FPy
	STS		FPACC1,FPx

	LD		FPy,Z+
	LDS		FPx,FPACC2
	ADC		FPx,FPy
	STS		FPACC2,FPx

	LD		FPy,Z+
	LDS		FPx,FPACC3
	ADC		FPx,FPy
	STS		FPACC3,FPx

	LD		FPy,Z+
	LDS		FPx,FPACC4
	ADC		FPx,FPy
	STS		FPACC4,FPx

	LD		FPy,Z+
	LDS		FPx,FPACC5
	ADC		FPx,FPy
	STS		FPACC5,FPx

	LD		FPy,Z+
	LDS		FPx,FPACC6
	ADC		FPx,FPy
	STS		FPACC6,FPx

	LD		FPy,Z+
	LDS		FPx,FPACC7
	ADC		FPx,FPy
	STS		FPACC7,FPx

	POP		FPy
	POP		FPx
	POP		FPN1
	OUT		SREG,temp
	POP		temp
	RET

FP_MAX:
; Liefert den greren Wert von AKKUMULATOR und FPN1
	PUSH	temp
	IN		temp,SREG
	PUSH	FPN1
	PUSH	FPx
  CALL  FP_COPYACC2B
  CALL  FP_NEG
  CALL  FP_ADD
	LDS		FPx,FPACC7 ; Vorzeichen vom Register A holen
	ROL		FPx
  BRCC  FP_MAX_FPN  
  CALL  FP_COPYB2ACC
	POP		FPx
	POP		FPN1
	OUT		SREG,temp
	POP		temp 
  RET

FP_MAX_FPN:
  CALL  FP_LOAD
	POP		FPx
	POP		FPN1
	OUT		SREG,temp
	POP		temp 
  RET

FP_MIN:
; Liefert den kleineren Wert von AKKUMULATOR und FPN1
	PUSH	temp
	IN		temp,SREG
	PUSH	FPN1
	PUSH	FPx
  CALL  FP_COPYACC2B
  CALL  FP_NEG
  CALL  FP_ADD
	LDS		FPx,FPACC7 ; Vorzeichen vom Register A holen
	ROL		FPx
  BRCC  FP_MIN_ACC
  CALL  FP_LOAD
	POP		FPx
	POP		FPN1
	OUT		SREG,temp
	POP		temp 
  RET

FP_MIN_ACC:
  CALL  FP_COPYB2ACC
	POP		FPx
	POP		FPN1
	OUT		SREG,temp
	POP		temp 
  RET

FP_MUL:
; Multipliziert den AKKUMULATOR mit FPN1
; 778 cycles, 48us
	PUSH	temp
	IN		temp,SREG
	PUSH	FPN1
	PUSH	FPA
	PUSH	FPB
	PUSH	FPC
	PUSH	FPx
	PUSH	FPy
	PUSH	FPz
	PUSH	DEVDATL
	PUSH	DEVDATH
	CALL	FP_SETFPN1_PTR

	LD		FPx,Z+
	STS		FPB0,FPx
	LD		FPx,Z+
	STS		FPB1,FPx
	LD		FPx,Z+
	STS		FPB2,FPx
	LD		FPx,Z+
	STS		FPB3,FPx
	LD		FPx,Z+
	STS		FPB4,FPx
	LD		FPx,Z+
	STS		FPB5,FPx
	LD		FPx,Z+
	STS		FPB6,FPx
	LD		FPx,Z+
	STS		FPB7,FPx

	MOV		FPz,ZERO

	LDS		FPx,FPB7 ; Vorzeichen vom Register B speichern
	ROL		FPx
	ROL		FPz
	LDS		FPx,FPACC7 ; Vorzeichen vom Register A speichern
	ROL		FPx
	ROL		FPz

	SBRC	FPz,1
	RCALL	FP_NEG_B
	SBRC	FPz,0
	RCALL	FP_NEG

	STS		FPE0,ZERO
	STS		FPE1,ZERO
	STS		FPE2,ZERO
	STS		FPE3,ZERO
	STS		FPE4,ZERO
	STS		FPE5,ZERO
	STS		FPE6,ZERO
	STS		FPE7,ZERO
	STS		FPE8,ZERO
	STS		FPE9,ZERO
	STS		FPE10,ZERO
	STS		FPE11,ZERO
	STS		FPE12,ZERO
	STS		FPE13,ZERO
	STS		FPE14,ZERO
	STS		FPE15,ZERO

	LDS		FPx,FPACC0
	LDS		FPy,FPB0
	MUL		FPx,FPy
	STS		FPE0,DEVDATL // E0 Fertig
	MOV		FPA,DEVDATH
	MOV		FPB,ZERO
	MOV		FPC,ZERO
; =========================================
	LDS		FPx,FPACC0
	LDS		FPy,FPB1
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC1
	LDS		FPy,FPB0
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO
	STS		FPE1,FPA	// E1 Fertig
	MOV		FPA,FPB
	MOV		FPB,FPC
	MOV		FPC,ZERO
; =========================================
	LDS		FPx,FPACC0
	LDS		FPy,FPB2
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC1
	LDS		FPy,FPB1
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC2
	LDS		FPy,FPB0
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO
	STS		FPE2,FPA	// E2 Fertig
	MOV		FPA,FPB
	MOV		FPB,FPC
	MOV		FPC,ZERO
; =========================================
	LDS		FPx,FPACC0
	LDS		FPy,FPB3
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC1
	LDS		FPy,FPB2
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC2
	LDS		FPy,FPB1
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC3
	LDS		FPy,FPB0
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDI		FPx,128		// Runden
	ADD		FPA,FPx
	ADC		FPB,ZERO
	ADC		FPC,ZERO
	STS		FPE3,FPA	// E3 Fertig
	MOV		FPA,FPB
	MOV		FPB,FPC
	MOV		FPC,ZERO
; =========================================
	LDS		FPx,FPACC0
	LDS		FPy,FPB4
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC1
	LDS		FPy,FPB3
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC2
	LDS		FPy,FPB2
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC3
	LDS		FPy,FPB1
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC4
	LDS		FPy,FPB0
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO
	STS		FPE4,FPA	// E4 Fertig
	MOV		FPA,FPB
	MOV		FPB,FPC
	MOV		FPC,ZERO
; =========================================
	LDS		FPx,FPACC0
	LDS		FPy,FPB5
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC1
	LDS		FPy,FPB4
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC2
	LDS		FPy,FPB3
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC3
	LDS		FPy,FPB2
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC4
	LDS		FPy,FPB1
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC5
	LDS		FPy,FPB0
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO
	STS		FPE5,FPA	// E5 Fertig
	MOV		FPA,FPB
	MOV		FPB,FPC
	MOV		FPC,ZERO
; =========================================
	LDS		FPx,FPACC0
	LDS		FPy,FPB6
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC1
	LDS		FPy,FPB5
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC2
	LDS		FPy,FPB4
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC3
	LDS		FPy,FPB3
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC4
	LDS		FPy,FPB2
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC5
	LDS		FPy,FPB1
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC6
	LDS		FPy,FPB0
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO
	STS		FPE6,FPA	// E6 Fertig
	MOV		FPA,FPB
	MOV		FPB,FPC
	MOV		FPC,ZERO
; =========================================
	LDS		FPx,FPACC0
	LDS		FPy,FPB7
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC1
	LDS		FPy,FPB6
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC2
	LDS		FPy,FPB5
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC3
	LDS		FPy,FPB4
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC4
	LDS		FPy,FPB3
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC5
	LDS		FPy,FPB2
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC6
	LDS		FPy,FPB1
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC7
	LDS		FPy,FPB0
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO
	STS		FPE7,FPA	// E7 Fertig
	MOV		FPA,FPB
	MOV		FPB,FPC
	MOV		FPC,ZERO
; =========================================
	LDS		FPx,FPACC1
	LDS		FPy,FPB7
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC2
	LDS		FPy,FPB6
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC3
	LDS		FPy,FPB5
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC4
	LDS		FPy,FPB4
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC5
	LDS		FPy,FPB3
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC6
	LDS		FPy,FPB2
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC7
	LDS		FPy,FPB1
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO
	STS		FPE8,FPA	// E8 Fertig
	MOV		FPA,FPB
	MOV		FPB,FPC
	MOV		FPC,ZERO
; =========================================
	LDS		FPx,FPACC2
	LDS		FPy,FPB7
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC3
	LDS		FPy,FPB6
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC4
	LDS		FPy,FPB5
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC5
	LDS		FPy,FPB4
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC6
	LDS		FPy,FPB3
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC7
	LDS		FPy,FPB2
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO
	STS		FPE9,FPA	// E9 Fertig
	MOV		FPA,FPB
	MOV		FPB,FPC
	MOV		FPC,ZERO
; =========================================
	LDS		FPx,FPACC3
	LDS		FPy,FPB7
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC4
	LDS		FPy,FPB6
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC5
	LDS		FPy,FPB5
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC6
	LDS		FPy,FPB4
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC7
	LDS		FPy,FPB3
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO
	STS		FPE10,FPA	// E10 Fertig
	MOV		FPA,FPB
	MOV		FPB,FPC
	MOV		FPC,ZERO
; =========================================
	LDS		FPx,FPACC4
	LDS		FPy,FPB7
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC5
	LDS		FPy,FPB6
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC6
	LDS		FPy,FPB5
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO

	LDS		FPx,FPACC7
	LDS		FPy,FPB4
	MUL		FPx,FPy
	ADD		FPA,DEVDATL
	ADC		FPB,DEVDATH
	ADC		FPC,ZERO
	STS		FPE11,FPA	// E11 Fertig
;	MOV		FPA,FPB
;	MOV		FPB,FPC
;	MOV		FPC,ZERO
; =========================================
;	LDS		FPx,FPACC5
;	LDS		FPy,FPB7
;	MUL		FPx,FPy
;	ADD		FPA,DEVDATL
;	ADC		FPB,DEVDATH
;	ADC		FPC,ZERO

;	LDS		FPx,FPACC6
;	LDS		FPy,FPB6
;	MUL		FPx,FPy
;	ADD		FPA,DEVDATL
;	ADC		FPB,DEVDATH
;	ADC		FPC,ZERO

;	LDS		FPx,FPACC7
;	LDS		FPy,FPB5
;	MUL		FPx,FPy
;	ADD		FPA,DEVDATL
;	ADC		FPB,DEVDATH
;	ADC		FPC,ZERO
;	STS		FPE12,FPA	// E12 Fertig
;	MOV		FPA,FPB
;	MOV		FPB,FPC
;	MOV		FPC,ZERO
; =========================================
;	LDS		FPx,FPACC6
;	LDS		FPy,FPB7
;	MUL		FPx,FPy
;	ADD		FPA,DEVDATL
;	ADC		FPB,DEVDATH
;	ADC		FPC,ZERO

;	LDS		FPx,FPACC7
;	LDS		FPy,FPB6
;	MUL		FPx,FPy
;	ADD		FPA,DEVDATL
;	ADC		FPB,DEVDATH
;	ADC		FPC,ZERO
;	STS		FPE13,FPA	// E13 Fertig
;	MOV		FPA,FPB
;	MOV		FPB,FPC
;	MOV		FPC,ZERO
; =========================================
;	LDS		FPx,FPACC7
;	LDS		FPy,FPB7
;	MUL		FPx,FPy
;	ADD		FPA,DEVDATL
;	ADC		FPB,DEVDATH
;	ADC		FPC,ZERO
;	STS		FPE14,FPA	// E14 Fertig
;	STS		FPE15,FPB	// E15 Fertig

	LDS		FPx,FPE4
	STS		FPACC0,FPx
	LDS		FPx,FPE5
	STS		FPACC1,FPx
	LDS		FPx,FPE6
	STS		FPACC2,FPx
	LDS		FPx,FPE7
	STS		FPACC3,FPx
	LDS		FPx,FPE8
	STS		FPACC4,FPx
	LDS		FPx,FPE9
	STS		FPACC5,FPx
	LDS		FPx,FPE10
	STS		FPACC6,FPx
	LDS		FPx,FPE11
	STS		FPACC7,FPx

	ANDI	FPz,0b00000011
	CPI		FPz,0b00000011
	BREQ	FP_MUL_NOTINVERT
	CPI		FPz,0b00000000
	BREQ	FP_MUL_NOTINVERT
	RCALL	FP_NEG
FP_MUL_NOTINVERT:
	POP		DEVDATH
	POP		DEVDATL
	POP		FPz
	POP		FPy
	POP		FPx
	POP		FPC
	POP		FPB
	POP		FPA
	POP		FPN1
	OUT		SREG,temp
	POP		temp
	RET
;============================================

;########################################
; Definition der Variablen im SRAM
;########################################

.DSEG
TXEP:		.BYTE	32
RXEP:		.BYTE	32
DTN:		.BYTE	1
TESTCOUNT:	.BYTE	1
B0:			.BYTE	1
B1:			.BYTE	1
B2:			.BYTE	1
B3:			.BYTE	1
B4:			.BYTE	1
B5:			.BYTE	1
B6:			.BYTE	1
B7:			.BYTE	1

; Buffer fr die Gertekonfiguration (Reglerparameter, Einstellungen)
S0:			.BYTE	1
S1:			.BYTE	1
S2:			.BYTE	1
S3:			.BYTE	1
S4:			.BYTE	1
S5:			.BYTE	1
S6:			.BYTE	1
S7:			.BYTE	1
SDONE:		.BYTE	1

; Speicher fr die 64Bit Fixed Point Zahlen
FPB0:		.BYTE	1
FPB1:		.BYTE	1
FPB2:		.BYTE	1
FPB3:		.BYTE	1
FPB4:		.BYTE	1
FPB5:		.BYTE	1
FPB6:		.BYTE	1
FPB7:		.BYTE	1

FPACC0:		.BYTE	1
FPACC1:		.BYTE	1
FPACC2:		.BYTE	1
FPACC3:		.BYTE	1
FPACC4:		.BYTE	1
FPACC5:		.BYTE	1
FPACC6:		.BYTE	1
FPACC7:		.BYTE	1

FPOVERRUN:	.BYTE	1

FPE0:		.BYTE	1
FPE1:		.BYTE	1
FPE2:		.BYTE	1
FPE3:		.BYTE	1
FPE4:		.BYTE	1
FPE5:		.BYTE	1
FPE6:		.BYTE	1
FPE7:		.BYTE	1
FPE8:		.BYTE	1
FPE9:		.BYTE	1
FPE10:		.BYTE	1
FPE11:		.BYTE	1
FPE12:		.BYTE	1
FPE13:		.BYTE	1
FPE14:		.BYTE	1
FPE15:		.BYTE	1

FPNUMBERS:	.BYTE 8*64
